import sys, math
sys.path[:0] = [".."]
from pyx import *
from pyx.normpath import normpathparam

def mark(c, t, phi, x, y):
    c.stroke(path.line(x-0.1, y, x+0.1, y))
    c.stroke(path.line(x, y-0.1, x, y+0.1))
    t = text.text(x, y, t, [text.halign.center, text.vshift.mathaxis])
    t.circlealign(0.2, math.cos(phi*math.pi/180), math.sin(phi*math.pi/180))
    c.insert(t)
c = canvas.canvas()
b = path.curve(0, 0, 0, 0.5, 0, 1, 4, 4).normpath()
l = path.line(*(b.atbegin() + b.atend()))
c.draw(l, [deco.shownormpath()])
c.draw(b, [deco.shownormpath()])
mark(c, "A", 180, 0, 0)
mark(c, "B", 180, 0, 0.5)
mark(c, "C", 180, 0, 1)
mark(c, "D", 0, 4, 4)
mark(c, "Y", -45, 2, 2)
mark(c, "X", 90, *b.at(normpathparam(b, 0, 0.5)))
mark(c, "X'", 135, *b.at(0.5*b.arclen()))
c.writeEPSfile("beziertoline")
c.writePDFfile("beziertoline")



# this code is really a mess ...

import sys
from Numeric import *
from RandomArray import *
from LinearAlgebra import *
from FFT import *
from functools import reduce


sqrt2 = math.sqrt(2)


def norm(v):
    return math.sqrt(reduce(lambda x, y: x+y*y, v, 0))

def blochvector(l, k):
    assert k >= 0 and k <= l/2
    v = cos(2*math.pi/l*(k)*arange(l))
    return v/norm(v)

def localvector(l, pos):
    v = zeros(l)
    v[pos] = 1
    return v/norm(v)

def local2vector(l, pos1, pos2):
    v = zeros(l)
    v[pos1] = 1
    v[pos2] = 1
    return v/norm(v)



def diag_anderson(l, w):

    """diagonalize the anderson model"""

    seed(x=1705, y=1111)
    pot=w*random(l)-0.5*w
    ham = zeros((l, l), Float)
    for i in range(l):
        ham[i, i] = pot[i]
        ham[i, (i+1)%l] = -1
        ham[i, (i-1)%l] = -1
    evalues, evectors = eigenvectors(ham)
    sorter = argsort(evalues)
    return take(evalues, sorter), take(evectors, sorter)



def diag_harper_full(l, lam):

    """diagonalize the harper model by full diagonalization"""

    def fibs(tonumber):
        result = [1, 1]
        while result[-1] < tonumber:
            result.append(result[-2]+result[-1])
        return result

    l2, ltest = fibs(l)[-2:]
    if ltest != l: raise ValueError("l isn't a fibonacci number")

    pot=lam*cos((2*pi*l2*arange(l))/l)
    ham = zeros((l, l), Float)
    for i in range(l):
        ham[i, i] = pot[i]
        ham[i, (i+1)%l] = 1
        ham[i, (i-1)%l] = 1
    evalues, evectors = eigenvectors(ham)
    sorter = argsort(evalues)
    return take(evalues, sorter), take(evectors, sorter)



def diag_harper(l, lam):

    """diagonalize the harper model by diagonalizing the symmetric and antisymmetic parts"""

    def fibs(tonumber):
        result = [1, 1]
        while result[-1] < tonumber:
            result.append(result[-2]+result[-1])
        return result

    def createsymm(dest, source):
        l = len(dest)
        if l % 2:
            dest[0] = source[0]
            for i in range(1, (l+1)/2):
                dest[i] = dest[l-i] = source[i] / sqrt2
        else:
            dest[0] = source[0]
            dest[l/2] = source[l/2]
            for i in range(1, l/2):
                dest[i] = dest[l-i] = source[i] / sqrt2

    def createasymm(dest, source):
        l = len(dest)
        if l % 2:
            dest[0] = 0
            for i in range(1, (l+1)/2):
                dest[i] = source[i-1] / sqrt2
                dest[l-i] = -source[i-1] / sqrt2
        else:
            dest[0] = dest[l/2] = 0
            for i in range(1, l/2):
                dest[i] = -source[i-1] / sqrt2
                dest[l-i] = source[i-1] / sqrt2

    l2, ltest = fibs(l)[-2:]
    if ltest != l: raise ValueError("l isn't a fibonacci number")

    # symmetric and antisymmetric diagonalization
    if l%2:
        # odd
        symmsize = (l-1)/2 + 1
        pot = lam*cos((2*pi*l2*arange(symmsize))/l)
        pot[symmsize - 1] += 1
        symmham = zeros((symmsize, symmsize), Float)
        for i in range(symmsize):
            symmham[i, i] = pot[i]
            if i > 1:
                symmham[i, i-1] = symmham[i-1, i] = 1
            elif i:
                symmham[i, i-1] = symmham[i-1, i] = sqrt2
    else:
        # even
        symmsize = l/2 + 1
        pot = lam*cos((2*pi*l2*arange(symmsize))/l)
        symmham = zeros((symmsize, symmsize), Float)
        for i in range(symmsize):
            symmham[i, i] = pot[i]
            if i > 1 and i < symmsize - 1:
                symmham[i, i-1] = symmham[i-1, i] = 1
            elif i:
                symmham[i, i-1] = symmham[i-1, i] = sqrt2
    symmevalues, symmevectors = eigenvectors(symmham)

    if l%2:
        # odd
        asymmsize = (l-1)/2
        pot = lam*cos((2*pi*l2*(arange(asymmsize)+1))/l)
        pot[asymmsize-1] -= 1
        asymmham = zeros((asymmsize, asymmsize), Float)
        for i in range(asymmsize):
            asymmham[i, i] = pot[i]
            if i:
                asymmham[i, i-1] = asymmham[i-1, i] = 1
    else:
        # even
        asymmsize = l/2 - 1
        pot = lam*cos((2*pi*l2*(arange(asymmsize)+1))/l)
        asymmham = zeros((asymmsize, asymmsize), Float)
        for i in range(asymmsize):
            asymmham[i, i] = pot[i]
            if i:
                asymmham[i, i-1] = asymmham[i-1, i] = 1
    asymmevalues, asymmevectors = eigenvectors(asymmham)

    # build complete solution
    symmsort = argsort(symmevalues)
    asymmsort = argsort(asymmevalues)
    j = k = 0
    evalues = zeros((l,), Float)
    evectors = zeros((l, l), Float)
    for i in range(l):
        if j != symmsize and (k == asymmsize or
                              symmevalues[symmsort[j]] < asymmevalues[asymmsort[k]]):
            evalues[i] = symmevalues[symmsort[j]]
            createsymm(evectors[i], symmevectors[symmsort[j]])
            j += 1
        else:
            evalues[i] = asymmevalues[asymmsort[k]]
            createasymm(evectors[i], asymmevectors[asymmsort[k]])
            k += 1
    return evalues, evectors



def test_harper(l, lam):

    """compare diag_harper and diag_harper_full"""

    res1 = diag_harper(l, lam)
    res2 = diag_harper_full(l, lam)
    for x1, x2 in zip(res1[0], res2[0]):
        assert math.fabs(x1-x2) < 1e-8, "eigenvalues differ"
    for v1, v2 in zip(res1[1], res2[1]):
        sum = 0
        for x1, x2 in zip(v1, v2):
            sum += x1 * x2
        if sum > 0:
            for x1, x2 in zip(v1, v2):
                assert math.fabs(x1-x2) < 1e-8, "eigenvectors differ\n%s\n%s" % (v1, v2)
        else:
            for x1, x2 in zip(v1, v2):
                assert math.fabs(x1+x2) < 1e-8, "eigenvectors differ\n%s\n%s" % (v1, -v2)

# test_harper(13, 5)



def wigner_slow(vector):

    """create wigner function of a state (direct way)"""

    l = len(vector)
    wig = zeros((l, l), Float)

    # wigner function (direct way)
    for x0loop in range(l):
        x0 = x0loop
        for k0loop in range(l):
            if l%2:
                k0 = (k0loop-0.5*l+0.5)*2*pi/l
            else:
                k0 = (k0loop-0.5*l+1)*2*pi/l
            sum = 0j
            for yloop in range(l):
                y = yloop - l/2 + 1 - l%2
                v = lambda x: vector[(x+10*l)%l]
                if y%2:
                    v1 = 0.5*(v(x0-(y-1)/2) + v(x0-(y+1)/2))
                    v2 = 0.5*(v(x0+(y-1)/2) + v(x0+(y+1)/2))
                #    v1 = 0
                #    v2 = 0
                else:
                    v1 = v(x0-y/2)
                    v2 = v(x0+y/2)
                sum += v1 * v2 * exp(1j*k0*y)
            assert abs(sum.imag) < 1e-10, "imaginary part should be zero"
            wig[x0loop, k0loop] = sum.real

    return wig



def wigner(vector):

    """create wigner function of a state (fft)"""

    l = len(vector)
    wig = zeros((l, l), Float)

    fftarray = zeros(l, Float)
    fftresult = zeros(l, Complex)
    wig = zeros((l, l), Float)
    for x0loop in range(l):
        x0 = x0loop
        for yloop in range(l):
            y = yloop - l/2 + 1 - l%2
            v = lambda x: vector[(x+10*l)%l]
            if y%2:
                v1 = 0.5*(v(x0-(y-1)/2) + v(x0-(y+1)/2))
                v2 = 0.5*(v(x0+(y-1)/2) + v(x0+(y+1)/2))
            #    v1 = 0
            #    v2 = 0
            else:
                v1 = v(x0-y/2)
                v2 = v(x0+y/2)
            fftarray[(y+10*l)%l] = v1 * v2
        fftresult = real_fft(fftarray)
        for k0loop in range(l):
            if l%2:
                index = int(abs(k0loop-0.5*l+0.5))
            else:
                index = int(abs(k0loop-0.5*l+1))
            wig[x0loop, k0loop] = fftresult[index].real

    return wig



def test_wigner(vector):

    """compare wigner_slow and wigner"""

    res1 = wigner_slow(vector)
    res2 = wigner(vector)
    for v1, v2 in zip(res1, res2):
        for x1, x2 in zip(v1, v2):
            assert math.fabs(x1-x2) < 1e-8, "wigner function differs\n%s\n%s" % (res1, res2)

# test_wigner(diag_anderson(10, 1)[1][5])
# test_wigner(diag_anderson(11, 1)[1][5])


def husimi_from_wigner(wig):

    """calculate the husimi function out of the wigner function"""

    l = len(wig)
    sigma = sqrt(l/pi/4)

    hus = zeros((l, l), Float)
    for x1 in range(l):
        for k1 in range(l):
            sys.stderr.write("wigner -> husimi (very slow) ...%6.2f%%\r" % ((100.0*(x1*l+k1))/l/l))
            hus[x1, k1] = 0
            for x2 in range(l):
                for k2 in range(l):
                    dx = x1-x2
                    if dx < -l/2: dx += l
                    if dx > l/2: dx -= l
                    dk = k1-k2
                    if dk < -l/2: dk += l
                    if dk > l/2: dk -= l
                    dk *= 2*pi/l
                    hus[x1, k1] += 2.0/l/l * wig[x2, k2] * exp(-0.5*dx*dx/sigma/sigma-2*sigma*sigma*dk*dk)
    sys.stderr.write("wigner -> husimi (very slow) ... done.   \n")

    return hus



def husimi_slow(vector):

    l = len(vector)
    hus = zeros((l, l), Float)
    phases = zeros((l, l), Float)
    sigma=sqrt(l/pi/4)
    factor=1/sqrt(sqrt(2*pi)*sigma*l)

    for x0loop in range(l):
        x0 = x0loop
        for k0loop in range(l):
            if l%2:
                k0 = (k0loop-0.5*l+0.5)*2*pi/l
            else:
                k0 = (k0loop-0.5*l+1)*2*pi/l
            sum = 0j
            for x in range(l):
                phase = exp(1j*k0*x)
                sum += vector[x] * factor * exp(-(x-x0-l)*(x-x0-l)/(4*sigma*sigma)) * phase
                sum += vector[x] * factor * exp(-(x-x0  )*(x-x0  )/(4*sigma*sigma)) * phase
                sum += vector[x] * factor * exp(-(x-x0+l)*(x-x0+l)/(4*sigma*sigma)) * phase
            hus[x0loop, k0loop] = abs(sum)*abs(sum)
            phases[x0loop, k0loop] = math.atan2(sum.imag, sum.real)

    return hus, phases



def husimi(vector):

    l = len(vector)
    hus = zeros((l, l), Float)
    sigma=sqrt(l/pi/4)
    factor=1/sqrt(sqrt(2*pi)*sigma*l)

    fftarray = zeros(l, Complex)
    fftresult = zeros(l, Complex)
    heights = zeros((l, l), Float)
    phases = zeros((l, l), Float)
    for x0loop in range(l):
        x0 = x0loop
        for xloop in range(l):
            x = xloop
            while (x-x0 < -0.5*l): x += l
            while (x-x0 > 0.5*l): x -=l
            fftarray[xloop] = vector[xloop] * factor * exp(-(x-x0)*(x-x0)/(4*sigma*sigma))
        #fftresult = real_fft(fftarray)
        fftresult = fft(fftarray)
        for k0loop in range(l):
            if l%2:
                index = (int(k0loop-0.5*l+0.5) + 10*l)%l
                #index = int(abs(k0loop-0.5*l+0.5))
            else:
                raise
                index = int(abs(k0loop-0.5*l+1))
            heights[x0loop, k0loop] = abs(fftresult[index])*abs(fftresult[index])
            phases[x0loop, k0loop] = math.atan2(fftresult[index].imag, fftresult[index].real)
            # if 2*x0loop > l: phases[x0loop, k0loop] = -phases[x0loop, k0loop] + math.pi
            # if 2*k0loop > l: phases[x0loop, k0loop] = -phases[x0loop, k0loop]

    return heights, phases



def test_husimi(vector):

    """compare husimi_slow and husimi"""

    res1 = husimi_slow(vector)
    res2 = husimi(vector)
    for v1, v2 in zip(res1, res2):
        for x1, x2 in zip(v1, v2):
            assert math.fabs(x1-x2) < 1e-8, "husimi function differs\n%s\n%s" % (res1, res2)

# test_husimi(diag_anderson(10, 1)[1][5])
# test_husimi(diag_anderson(11, 1)[1][5])

l=101
values, vectors = diag_anderson(l, 0.5)
vector = vectors[24]
heights, phases = husimi(vector)

f = open("example.dat", "w")
for x in range(l):
    for y in range(l):
        f.write("%i %i %+20.15e %+20.15e\n" % (x, y, heights[x, y], phases[x, y]))



#!/usr/bin/env python
import sys, math
sys.path[:0] = [".."]
import pyx
from pyx import *

c = canvas.canvas()
c.stroke(path.line(0, 0, 4, 2), [deco.earrow.Large()])
t = text.text(2, 1, r"$\vec p$")
t.linealign(0.1, -2/math.sqrt(20), 4/math.sqrt(20))
c.insert(t)
c.stroke(path.line(0, 0, 1, 1), [deco.earrow.Large()])
t = text.text(0.5, 0.5, r"$\vec a$")
t.linealign(0.1, -1/math.sqrt(2), 1/math.sqrt(2))
c.insert(t)
c.stroke(path.line(-1, 3, 3, -1), [style.linestyle.dotted])
c.stroke(path.line(1, 1, 1-1/math.sqrt(2), 1+1/math.sqrt(2)), [deco.earrow.Large()])
t = text.text(1-1/math.sqrt(8), 1+1/math.sqrt(8), r"$\vec b$")
t.linealign(0.1, 1/math.sqrt(2), 1/math.sqrt(2))
c.insert(t)
c.writeEPSfile("boxalignpal")

###############################################################################

c = canvas.canvas()
c.stroke(path.line(0, 0, 4, 2), [deco.earrow.Large()])
t = text.text(2, 1, r"$\vec p$")
t.linealign(0.1, -2/math.sqrt(20), 4/math.sqrt(20))
c.insert(t)
c.stroke(path.line(0, 0, -1, 1), [deco.earrow.Large()])
t = text.text(-0.5, 0.5, r"$\vec a$")
t.linealign(0.1, -1/math.sqrt(2), -1/math.sqrt(2))
c.insert(t)
c.stroke(path.path(path.arc(0, 0, math.sqrt(2), 0, 360), path.closepath()), [style.linestyle.dotted])
c.writeEPSfile("boxalignpac")

###############################################################################

c = canvas.canvas()
c.stroke(path.line(0, 0, 4, 2), [deco.earrow.Large()])
t = text.text(2, 1, r"$\vec e$")
t.linealign(0.1, -2/math.sqrt(20), 4/math.sqrt(20))
c.insert(t)
c.stroke(path.line(0, 0, 6, 2), [deco.earrow.Large()])
t = text.text(3, 1, r"$\vec f$")
t.linealign(0.1, 2/math.sqrt(60), -6/math.sqrt(60))
c.insert(t)
c.stroke(path.line(6, 2, 4, 2), [deco.earrow.Large()])
t = text.text(5, 2, r"$\vec g$")
t.linealign(0.1, 0, 1)
c.insert(t)
c.stroke(path.line(0, 0, -1, 1), [deco.earrow.Large()])
t = text.text(-0.5, 0.5, r"$\vec a$")
t.linealign(0.1, -1/math.sqrt(2), -1/math.sqrt(2))
c.insert(t)
c.stroke(path.path(path.arc(0, 0, math.sqrt(2), 0, 360), path.closepath()), [style.linestyle.dotted])
c.stroke(path.line(0, 0, 0, math.sqrt(2)), [deco.earrow.Large()])
t = text.text(0, 1/math.sqrt(2), r"$\vec b$")
t.linealign(0.1, -1, 0)
c.insert(t)
c.writeEPSfile("boxalignlac")

###############################################################################

def drawexample(canvas, corner, linealign):
    r = 1.5
    canvas.stroke(path.path(path.arc(0, 0, r, 0, 360)))
    phi = 0
    while phi < 2 * math.pi + 1e-10:
        if corner:
            b = box.polygon(center=(0, 0), corners=[(0, 0), (1, 0), (0.5, math.sqrt(3)/2)])
        else:
            b = box.polygon(center=(0, 0), corners=[(-0.5, -math.sqrt(3)/6), (0.5, -math.sqrt(3)/6), (0, math.sqrt(3)/3)])
        if linealign:
            b.linealign(r, math.cos(phi), math.sin(phi))
        else:
            b.circlealign(r, math.cos(phi), math.sin(phi))
        if round(phi / math.pi * 2 * 100) % 100:
            canvas.stroke(b.path())
        else:
            canvas.stroke(b.path(), [color.rgb.red])
        phi += math.pi / 50

d = 6
c = canvas.canvas()
sc = c.insert(canvas.canvas([trafo.translate(0, d)]))
drawexample(sc, 0, 0)
sc = c.insert(canvas.canvas([trafo.translate(d, d)]))
drawexample(sc, 0, 1)
sc = c.insert(canvas.canvas([trafo.translate(0, 0)]))
drawexample(sc, 1, 0)
sc = c.insert(canvas.canvas([trafo.translate(d, 0)]))
drawexample(sc, 1, 1)
c.text(0, 1.5*d, "align at a circle", [text.halign.center])
c.text(d, 1.5*d, "align at tangents", [text.halign.center])
c.text(-0.5*d, d, "reference point at the triangle center", [text.halign.center, trafo.rotate(90)])
c.text(-0.5*d, 0, "reference point at a triangle corner", [text.halign.center, trafo.rotate(90)])
c.writeEPSfile("boxalignexample")





from itertools import product, combinations
from operator import eq
from pyx import *

projector = graph.graphxyz.central(10, 20, -30).point

a = 2
cube = list(product([-a, a], repeat=3))
edges = list(filter(lambda edge: sum(map(eq, edge[0], edge[1])) == 2,
                    combinations(cube, 2)))

c = canvas.canvas()

for begin, end in edges:
    x1, y1 = projector(*begin)
    x2, y2 = projector(*end)
    if begin[0] != end[0]:
        col = color.rgb.blue
    elif begin[0] == a:
        col = color.rgb.red
    else:
        col = color.rgb.green
    c.stroke(path.line(x1, y1, x2, y2), [style.linewidth.Thick, col])

for vertex in cube:
    x, y = projector(*vertex)
    c.fill(path.circle(x, y, 0.2))

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



from pyx import *

textpath = text.text(0, 0, r"\PyX").textpath().reversed()
decotext = r"\PyX{} is fun! "*50
scale = text.text(0, 0, decotext).width/textpath.arclen()

c = canvas.canvas()
c.draw(textpath, [trafo.scale(scale),
                  deco.filled([color.gray(0.5)]),
                  deco.curvedtext(decotext)])
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import math
from pyx import *

circle = path.circle(0, 0, 5/math.pi)
opencircle, = circle.split(circle.begin())

c = canvas.canvas()

def showsplit(x, y, path, splitpoints, description):
    c.text(x, y, description, [text.halign.center, text.vshift.mathaxis])
    segments = path.transformed(trafo.translate(x, y)).split(splitpoints)
    print("path sement lengths for ``%s'':" % description)
    for segment, segment_color in zip(segments, [color.rgb.red, color.rgb.green, color.rgb.blue]):
        print("  %s" % segment.arclen())
        c.stroke(segment, [deco.earrow.normal, style.linestyle.dashed, segment_color])
    print()

showsplit(0, 0, opencircle, [opencircle.begin()+1, opencircle.end()-1], "open, $p_1 < p_2$")
showsplit(0, -5, opencircle, [opencircle.end()-1, opencircle.begin()+1], "open, $p_1 > p_2$")

showsplit(5, 0, circle, [circle.begin()+1, circle.end()-1], "closed, $p_1 < p_2$")
showsplit(5, -5, circle, [circle.end()-1, circle.begin()+1], "closed, $p_1 > p_2$")

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

from pyx import *
from pyx.path import *
from pyx.normpath import normpathparam
import math
set(epsilon=1e-7)

class NormpathTestCase(unittest.TestCase):

    def assertAlmostEqualNormsubpathitem(self, nspi1, nspi2):
        if isinstance(nspi1, normline_pt):
            assert isinstance(nspi2, normline_pt), "%s != %s" % (nspi1, nspi2)
            self.assertAlmostEqual(nspi1.x0_pt, nspi2.x0_pt)
            self.assertAlmostEqual(nspi1.y0_pt, nspi2.y0_pt)
            self.assertAlmostEqual(nspi1.x1_pt, nspi2.x1_pt)
            self.assertAlmostEqual(nspi1.y1_pt, nspi2.y1_pt)
        else:
            assert isinstance(nspi1, normcurve_pt), "%s != %s" % (nspi1, nspi2)
            assert isinstance(nspi2, normcurve_pt), "%s != %s" % (nspi1, nspi2)
            self.assertAlmostEqual(nspi1.x0_pt, nspi2.x0_pt)
            self.assertAlmostEqual(nspi1.y0_pt, nspi2.y0_pt)
            self.assertAlmostEqual(nspi1.x1_pt, nspi2.x1_pt)
            self.assertAlmostEqual(nspi1.y1_pt, nspi2.y1_pt)
            self.assertAlmostEqual(nspi1.x2_pt, nspi2.x2_pt)
            self.assertAlmostEqual(nspi1.y2_pt, nspi2.y2_pt)
            self.assertAlmostEqual(nspi1.x3_pt, nspi2.x3_pt)
            self.assertAlmostEqual(nspi1.y3_pt, nspi2.y3_pt)

    def assertAlmostEqualNormsubpath(self, nsp1, nsp2):
        assert len(nsp1) == len(nsp2), "%s != %s" % (nsp1, nsp2)
        assert nsp1.closed == nsp2.closed, "%s != %s" % (nsp1, nsp2)
        for nspi1, nspi2 in zip(nsp1, nsp2):
            self.assertAlmostEqualNormsubpathitem(nspi1, nspi2)

    def assertAlmostEqualNormpath(self, np1, np2):
        assert len(np1) == len(np2), "%s != %s" % (np1, np2)
        for nsp1, nsp2 in zip(np1, np2):
            self.assertAlmostEqualNormsubpath(nsp1, nsp2)

    def testparam(self):
        p = ( normpath([normsubpath([normline_pt(0, 0, 10, 0),
                                   normline_pt(10, 0, 10, 20),
                                   normline_pt(10, 20, 0, 20),
                                   normline_pt(0, 20, 0, 0)], closed=1)]) +
              circle_pt(0, 0, 10) +
              line_pt(0, 0, 2, 0))

        param = normpathparam(p, 0, 1.5)
        param = param + 0
        self.assertEqual(param.normsubpathindex, 0)
        self.assertAlmostEqual(param.normsubpathparam, 1.5)
        param = param + 15 * unit.t_pt
        self.assertEqual(param.normsubpathindex, 0)
        self.assertAlmostEqual(param.normsubpathparam, 2.5)
        param += 24.9 * unit.t_pt
        self.assertEqual(param.normsubpathindex, 0)
        self.assertAlmostEqual(param.normsubpathparam, 3.995)
        param = 0.1 * unit.t_pt + param
        self.assertEqual(param.normsubpathindex, 1)
        self.assertAlmostEqual(param.normsubpathparam, 0)
        param = param + 0.5*circle_pt(0, 0, 10).arclen()
        circlerange = len(p.normsubpaths[1])
        self.assertEqual(param.normsubpathindex, 1)
        self.assertAlmostEqual(param.normsubpathparam, 0.5*circlerange, 4)
        param = param + 0.5*circle_pt(0, 0, 10).arclen()
        param = param + 2 * unit.t_pt
        self.assertEqual(param.normsubpathindex, 2)
        self.assertAlmostEqual(param.normsubpathparam, 1, 4)
        param = param + 1 * unit.t_pt
        self.assertEqual(param.normsubpathindex, 2)
        self.assertAlmostEqual(param.normsubpathparam, 1.5, 4)

        param = normpathparam(p, 0, 1.5)
        param = param - 15 * unit.t_pt
        self.assertEqual(param.normsubpathindex, 0)
        self.assertAlmostEqual(param.normsubpathparam, 0.5)
        param -= 10 * unit.t_pt
        self.assertEqual(param.normsubpathindex, 0)
        self.assertAlmostEqual(param.normsubpathparam, -0.5)
        
        param = normpathparam(p, 0, 1.2)
        param2 = 2*param
        param += param
        self.assertEqual(param.normsubpathindex, param2.normsubpathindex)
        self.assertEqual(param.normsubpathparam, param2.normsubpathparam)

        param = normpathparam(p, 0, 1.2)
        self.assertTrue(param < 15 * unit.t_pt)
        self.assertTrue(15 * unit.t_pt > param)
        self.assertTrue(param > 12 * unit.t_pt)
        self.assertTrue(12 * unit.t_pt < param)
        self.assertTrue(param < 1)
        self.assertTrue(1 > param)

    def testat(self):
        p = normpath([normsubpath([normline_pt(0, 0, 10, 0),
                                   normline_pt(10, 0, 10, 20),
                                   normline_pt(10, 20, 0, 20),
                                   normline_pt(0, 20, 0, 0)], closed=1)])
        params = [-5, 0, 5, 10, 20, 30, 35, 40, 50, 60, 70]
        ats = (-5, 0), (0, 0), (5, 0), (10, 0), (10, 10), (10, 20), (5, 20), (0, 20), (0, 10), (0, 0), (0, -10)
        for param, (at_x, at_y) in zip(params, ats):
            self.assertAlmostEqual(p.at_pt(param)[0], at_x)
            self.assertAlmostEqual(p.at_pt(param)[1], at_y)
        for (at_x, at_y), (at2_x, at2_y) in zip(p.at_pt(params), ats):
            self.assertAlmostEqual(at_x, at2_x)
            self.assertAlmostEqual(at_y, at2_y)
        p = normpath([normsubpath([normline_pt(0, 0, 3, 0),
                                   normcurve_pt(3, 0, 3, 2, 3, 4, 3, 6),
                                   normcurve_pt(3, 6, 2, 6, 1, 6, 0, 6),
                                   normline_pt(0, 6, 0, 0)], closed=1)])
        self.assertAlmostEqual(p.at_pt(6)[0], 3)
        self.assertAlmostEqual(p.at_pt(6)[1], 3)
        self.assertAlmostEqual(p.at_pt(4.5)[0], 3)
        self.assertAlmostEqual(p.at_pt(4.5)[1], 1.5)

    def testarclentoparam(self):
        p = ( normpath([normsubpath([normline_pt(0, 0, 10, 0),
                                   normline_pt(10, 0, 10, 20),
                                   normline_pt(10, 20, 0, 20),
                                   normline_pt(0, 20, 0, 0)], closed=1)]) +
              circle_pt(0, 0, 10) +
              line_pt(0, 0, 2, 0))

        arclens_pt = [20, 30, -2, 61, 100, 200, -30, 1000]
        for arclen_pt, arclen2_pt in zip(arclens_pt, p.paramtoarclen_pt(p.arclentoparam_pt(arclens_pt))):
            self.assertAlmostEqual(arclen_pt, arclen2_pt, 4)

        arclens = [x*unit.t_pt for x in [20, 30, -2, 61, 100, 200, -30, 1000]]
        for arclen, arclen2 in zip(arclens, p.paramtoarclen(p.arclentoparam(arclens))):
            self.assertAlmostEqual(unit.tom(arclen), unit.tom(arclen2), 4)

    def testsplit(self):
        p = normline_pt(0, 0, 10, 0)
        self.assertRaises(ValueError, p.segments, [])
        self.assertRaises(ValueError, p.segments, [0.2])
        s = p.segments([0.2, 0.8])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpathitem(s[0], normline_pt(2, 0, 8, 0))
        s = p.segments([-0.2, 1.8])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpathitem(s[0], normline_pt(-2, 0, 18, 0))

        p = normcurve_pt(0, 0, 10, 0, 20, 0, 30, 0)
        self.assertRaises(ValueError, p.segments, [])
        self.assertRaises(ValueError, p.segments, [0.2])
        s = p.segments([0.2, 0.8])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpathitem(s[0], normcurve_pt(6, 0, 12, 0, 18, 0, 24, 0))
        s = p.segments([-0.2, 1.8])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpathitem(s[0], normcurve_pt(-6, 0, 14, 0, 34, 0, 54, 0))

        p = normsubpath([normline_pt(0, 0, 1, 0),
                         normline_pt(1, 0, 1, 1),
                         normline_pt(1, 1, 0, 1),
                         normline_pt(0, 1, 0, 0)])
        self.assertRaises(ValueError, p.segments, [])
        self.assertRaises(ValueError, p.segments, [0.2])
        s = p.segments([0.5, 2.5])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpath(s[0], normsubpath([normline_pt(0.5, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0.5, 1)]))
        s = p.segments([4.5, -1])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpath(s[0], normsubpath([normline_pt(0, -0.5, 0, 1), normline_pt(0, 1, 1, 1), normline_pt(1, 1, 1, 0), normline_pt(1, 0, -1, 0)]))
        s = p.segments([0, 0.1, 1.2, 1.3, 3.4, 0.5, 4])
        self.assertEqual(len(s), 6)
        self.assertAlmostEqualNormsubpath(s[0], normsubpath([normline_pt(0, 0, 0.1, 0)]))
        self.assertAlmostEqualNormsubpath(s[1], normsubpath([normline_pt(0.1, 0, 1, 0), normline_pt(1, 0, 1, 0.2)]))
        self.assertAlmostEqualNormsubpath(s[2], normsubpath([normline_pt(1, 0.2, 1, 0.3)]))
        self.assertAlmostEqualNormsubpath(s[3], normsubpath([normline_pt(1, 0.3, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0.6)]))
        self.assertAlmostEqualNormsubpath(s[4], normsubpath([normline_pt(0, 0.6, 0, 1), normline_pt(0, 1, 1, 1), normline_pt(1, 1, 1, 0), normline_pt(1, 0, 0.5, 0)]))
        self.assertAlmostEqualNormsubpath(s[5], normsubpath([normline_pt(0.5, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0)]))

        p = normsubpath([normline_pt(0, 0, 1, 0),
                         normline_pt(1, 0, 1, 1),
                         normline_pt(1, 1, 0, 1)], closed=1)
        self.assertRaises(ValueError, p.segments, [])
        self.assertRaises(ValueError, p.segments, [0.5])
        s = p.segments([0.5, 2.5])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpath(s[0], normsubpath([normline_pt(0.5, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0.5, 1)]))
        s = p.segments([4.5, -1])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormsubpath(s[0], normsubpath([normline_pt(0, -0.5, 0, 1), normline_pt(0, 1, 1, 1), normline_pt(1, 1, 1, 0), normline_pt(1, 0, -1, 0)]))
        s = p.segments([0, 0.1, 1.2, 1.3, 3.4, 0.5, 4])
        self.assertEqual(len(s), 5)
        self.assertAlmostEqualNormsubpath(s[0], normsubpath([normline_pt(0.5, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0), normline_pt(0, 0, 0.1, 0)]))
        self.assertAlmostEqualNormsubpath(s[1], normsubpath([normline_pt(0.1, 0, 1, 0), normline_pt(1, 0, 1, 0.2)]))
        self.assertAlmostEqualNormsubpath(s[2], normsubpath([normline_pt(1, 0.2, 1, 0.3)]))
        self.assertAlmostEqualNormsubpath(s[3], normsubpath([normline_pt(1, 0.3, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0.6)]))
        self.assertAlmostEqualNormsubpath(s[4], normsubpath([normline_pt(0, 0.6, 0, 1), normline_pt(0, 1, 1, 1), normline_pt(1, 1, 1, 0), normline_pt(1, 0, 0.5, 0)]))
        s = p.segments([4, 0.1, 1.2, 1.3, 3.4, 0.5, 0])
        self.assertEqual(len(s), 5)
        self.assertAlmostEqualNormsubpath(s[0], normsubpath([normline_pt(0.5, 0, 0, 0), normline_pt(0, 0, 0, 1), normline_pt(0, 1, 1, 1), normline_pt(1, 1, 1, 0), normline_pt(1, 0, 0.1, 0)]))
        self.assertAlmostEqualNormsubpath(s[1], normsubpath([normline_pt(0.1, 0, 1, 0), normline_pt(1, 0, 1, 0.2)]))
        self.assertAlmostEqualNormsubpath(s[2], normsubpath([normline_pt(1, 0.2, 1, 0.3)]))
        self.assertAlmostEqualNormsubpath(s[3], normsubpath([normline_pt(1, 0.3, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0.6)]))
        self.assertAlmostEqualNormsubpath(s[4], normsubpath([normline_pt(0, 0.6, 0, 1), normline_pt(0, 1, 1, 1), normline_pt(1, 1, 1, 0), normline_pt(1, 0, 0.5, 0)]))

        p = normpath([normsubpath([normline_pt(0, 0, 1, 0), normline_pt(1, 0, 2, 0), normline_pt(2, 0, 3, 0), normline_pt(3, 0, 4, 0)]),
                      normsubpath([normline_pt(0, 1, 1, 1), normline_pt(1, 1, 2, 1), normline_pt(2, 1, 3, 1), normline_pt(3, 1, 4, 1)]),
                      normsubpath([normline_pt(0, 2, 1, 2), normline_pt(1, 2, 2, 2), normline_pt(2, 2, 3, 2), normline_pt(3, 2, 4, 2)]),
                      normsubpath([normline_pt(0, 3, 1, 3), normline_pt(1, 3, 2, 3), normline_pt(2, 3, 3, 3), normline_pt(3, 3, 4, 3)]),
                      normsubpath([normline_pt(0, 4, 1, 4), normline_pt(1, 4, 2, 4), normline_pt(2, 4, 3, 4), normline_pt(3, 4, 4, 4)])])
        s = p.split([normpathparam(p, 3, 0.1),
                     normpathparam(p, 4, 1.2),
                     normpathparam(p, 2, 2.3),
                     normpathparam(p, 0, 3.4),
                     normpathparam(p, 1, 0.5)])
        self.assertEqual(len(s), 6)
        self.assertAlmostEqualNormpath(s[0], normpath([normsubpath([normline_pt(0, 0, 1, 0), normline_pt(1, 0, 2, 0), normline_pt(2, 0, 3, 0), normline_pt(3, 0, 4, 0)]),
                                                           normsubpath([normline_pt(0, 1, 1, 1), normline_pt(1, 1, 2, 1), normline_pt(2, 1, 3, 1), normline_pt(3, 1, 4, 1)]),
                                                           normsubpath([normline_pt(0, 2, 1, 2), normline_pt(1, 2, 2, 2), normline_pt(2, 2, 3, 2), normline_pt(3, 2, 4, 2)]),
                                                           normsubpath([normline_pt(0, 3, 0.1, 3)])]))
        self.assertAlmostEqualNormpath(s[1], normpath([normsubpath([normline_pt(0.1, 3, 1, 3), normline_pt(1, 3, 2, 3), normline_pt(2, 3, 3, 3), normline_pt(3, 3, 4, 3)]),
                                                           normsubpath([normline_pt(0, 4, 1, 4), normline_pt(1, 4, 1.2, 4)])]))
        self.assertAlmostEqualNormpath(s[2], normpath([normsubpath([normline_pt(1.2, 4, 1, 4), normline_pt(1, 4, 0, 4)]),
                                                           normsubpath([normline_pt(4, 3, 3, 3), normline_pt(3, 3, 2, 3), normline_pt(2, 3, 1, 3), normline_pt(1, 3, 0, 3)]),
                                                           normsubpath([normline_pt(4, 2, 3, 2), normline_pt(3, 2, 2.3, 2)])]))
        self.assertAlmostEqualNormpath(s[3], normpath([normsubpath([normline_pt(2.3, 2, 2, 2), normline_pt(2, 2, 1, 2), normline_pt(1, 2, 0, 2)]),
                                                           normsubpath([normline_pt(4, 1, 3, 1), normline_pt(3, 1, 2, 1), normline_pt(2, 1, 1, 1), normline_pt(1, 1, 0, 1)]),
                                                           normsubpath([normline_pt(4, 0, 3.4, 0)])]))
        self.assertAlmostEqualNormpath(s[4], normpath([normsubpath([normline_pt(3.4, 0, 4, 0)]),
                                                           normsubpath([normline_pt(0, 1, 0.5, 1)])]))
        self.assertAlmostEqualNormpath(s[5], normpath([normsubpath([normline_pt(0.5, 1, 1, 1), normline_pt(1, 1, 2, 1), normline_pt(2, 1, 3, 1), normline_pt(3, 1, 4, 1)]),
                                                           normsubpath([normline_pt(0, 2, 1, 2), normline_pt(1, 2, 2, 2), normline_pt(2, 2, 3, 2), normline_pt(3, 2, 4, 2)]),
                                                           normsubpath([normline_pt(0, 3, 1, 3), normline_pt(1, 3, 2, 3), normline_pt(2, 3, 3, 3), normline_pt(3, 3, 4, 3)]),
                                                           normsubpath([normline_pt(0, 4, 1, 4), normline_pt(1, 4, 2, 4), normline_pt(2, 4, 3, 4), normline_pt(3, 4, 4, 4)])]))

        p = normpath([normsubpath([normline_pt(0, -5, 1, -5)]),
                      normsubpath([normline_pt(0, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1)], closed=1),
                      normsubpath([normline_pt(0, 5, 1, 5)])])
        s = p.split([normpathparam(p, 1, 3.5)])
        self.assertEqual(len(s), 1)
        self.assertAlmostEqualNormpath(s[0], normpath([normsubpath([normline_pt(0, -5, 1, -5)]),
                                                           normsubpath([normline_pt(0, 0.5, 0, 0), normline_pt(0, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0.5)]),
                                                           normsubpath([normline_pt(0, 5, 1, 5)])]))

        p = normpath([normsubpath([normline_pt(0, -5, 1, -5)]),
                      normsubpath([normline_pt(0, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1)], closed=1),
                      normsubpath([normline_pt(0, 5, 1, 5)])])
        s = p.split([normpathparam(p, 0, 0.5),
                     normpathparam(p, 1, 3.5),
                     normpathparam(p, 0, 0.5),
                     normpathparam(p, 2, 0.5)])
        self.assertEqual(len(s), 5)
        self.assertAlmostEqualNormpath(s[0], normpath([normsubpath([normline_pt(0, -5, 0.5, -5)])]))
        self.assertAlmostEqualNormpath(s[1], normpath([normsubpath([normline_pt(0.5, -5, 1, -5)]),
                                                           normsubpath([normline_pt(0, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0.5)])]))
        # XXX should we do ???: normsubpath([normline_pt(0, 0.5, 0, 0), normline_pt(0, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0.5)])]))
        # same question in the next line ...
        self.assertAlmostEqualNormpath(s[2], normpath([normsubpath([normline_pt(0, 0.5, 0, 1), normline_pt(0, 1, 1, 1), normline_pt(1, 1, 1, 0), normline_pt(1, 0, 0, 0)]),
                                                           normsubpath([normline_pt(1, -5, 0.5, -5)])]))
        self.assertAlmostEqualNormpath(s[3], normpath([normsubpath([normline_pt(0.5, -5, 1, -5)]),
                                                           normsubpath([normline_pt(0, 0, 1, 0), normline_pt(1, 0, 1, 1), normline_pt(1, 1, 0, 1), normline_pt(0, 1, 0, 0)], closed=1),
                                                           normsubpath([normline_pt(0, 5, 0.5, 5)])]))
        self.assertAlmostEqualNormpath(s[4], normpath([normsubpath([normline_pt(0.5, 5, 1, 5)])]))

    def testshortnormsubpath(self):
        sp = normsubpath(epsilon=1)
        sp.append(normline_pt(0, 0, 0.5, 0))
        sp.append(normline_pt(0.5, 0, 1.5, 0))

        sp.append(normline_pt(1.5, 0, 1.5, 0.3))
        sp.append(normline_pt(1.5, 0.3, 1.5, 0.6))
        sp.append(normline_pt(1.5, 0.6, 1.5, 0.9))
        sp.append(normline_pt(1.5, 0.9, 1.5, 1.2))

        sp.append(normline_pt(1.5, 1.2, 1.3, 1.6))
        sp.append(normcurve_pt(1.3, 1.6, 1.4, 1.7, 1.3, 1.7, 1.3, 1.8))
        sp.append(normcurve_pt(1.3, 1.8, 4.4, 2.7, 5.3, 5.7, 1.4, 1.8))

        self.assertAlmostEqualNormsubpath(sp, normsubpath([normline_pt(0, 0, 1.5, 0), normline_pt(1.5, 0, 1.5, 1.2), normcurve_pt(1.5, 1.2, 4.4, 2.7, 5.3, 5.7, 1.4, 1.8)]))

    def testcuspremoval(self):
        self.assertAlmostEqualNormsubpath(normsubpath([normcurve_pt(0, 0, 5, 5, 0, 5, 5, 0)], epsilon=1),
                                          normsubpath([normcurve_pt(0, 0, 2.5, 2.5, 2.5, 3.405172413793103, 2.5, 3.75), normcurve_pt(2.5, 3.75, 2.5, 3.405172413793103, 2.5, 2.5, 5, 0)], epsilon=None))

    def testintersectnormsubpath(self):
        smallposy = 0.09
        smallnegy = -0.01
        p1 = normsubpath([normline_pt(-1, 0, 1, 0)])
        p2 = normsubpath([normline_pt(0, smallposy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, 1+smallnegy),
                          normline_pt(0, 1+smallnegy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, smallposy)], closed=0)
        p1.epsilon = p2.epsilon = 0.05
        intersect = p2.intersect(p1)
        self.assertEqual(len(intersect[0]), 2)
        self.assertAlmostEqual(intersect[0][0], 0.9)
        self.assertAlmostEqual(intersect[0][1], 2.99)

        smallposy = 0.09
        smallnegy = -0.01
        p1 = normsubpath([normline_pt(-1, 0, 1, 0)])
        p2 = normsubpath([normline_pt(0, smallposy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, 1+smallnegy),
                          normline_pt(0, 1+smallnegy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, smallposy)], closed=1)
        p1.epsilon = p2.epsilon = 0.05
        intersect = p2.intersect(p1)
        self.assertEqual(len(intersect[0]), 2)
        self.assertAlmostEqual(intersect[0][0], 0.9)
        self.assertAlmostEqual(intersect[0][1], 2.99)

        smallposy = 0.01
        smallnegy = -0.09
        p1 = normsubpath([normline_pt(-1, 0, 1, 0)])
        p2 = normsubpath([normline_pt(0, smallposy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, 1+smallnegy),
                          normline_pt(0, 1+smallnegy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, smallposy)], closed=0)
        p1.epsilon = p2.epsilon = 0.05
        intersect = p2.intersect(p1)
        self.assertEqual(len(intersect[0]), 4)
        self.assertAlmostEqual(intersect[0][0], 0.1)
        self.assertAlmostEqual(intersect[0][1], 1.09)
        self.assertAlmostEqual(intersect[0][2], 2.91)
        self.assertAlmostEqual(intersect[0][3], 3.9)

        smallposy = 0.01
        smallnegy = -0.09
        p1 = normsubpath([normline_pt(-1, 0, 1, 0)])
        p2 = normsubpath([normline_pt(0, smallposy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, 1+smallnegy),
                          normline_pt(0, 1+smallnegy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, smallposy)], closed=1)
        p1.epsilon = p2.epsilon = 0.05
        intersect = p2.intersect(p1)
        self.assertEqual(len(intersect[0]), 3)
        self.assertAlmostEqual(intersect[0][0], 0.1)
        self.assertAlmostEqual(intersect[0][1], 1.09)
        self.assertAlmostEqual(intersect[0][2], 2.91)

        smallposy = 0.01
        smallnegy = -0.01
        p1 = normsubpath([normline_pt(-1, 0, 1, 0)])
        p2 = normsubpath([normline_pt(0, smallposy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, 1+smallnegy),
                          normline_pt(0, 1+smallnegy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, smallposy)], closed=0)
        p1.epsilon = p2.epsilon = 0.05
        intersect = p2.intersect(p1)
        self.assertEqual(len(intersect[0]), 2)
        self.assertAlmostEqual(intersect[0][0], 0.5)
        self.assertAlmostEqual(intersect[0][1], 2.99)

        smallposy = 0.01
        smallnegy = -0.01
        p1 = normsubpath([normline_pt(-1, 0, 1, 0)])
        p2 = normsubpath([normline_pt(0, smallposy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, 1+smallnegy),
                          normline_pt(0, 1+smallnegy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, smallposy)], closed=1)
        p1.epsilon = p2.epsilon = 0.05
        intersect = p2.intersect(p1)
        self.assertEqual(len(intersect[0]), 1)
        self.assertAlmostEqual(intersect[0][0], 0.5)

        smallposy = 0.1
        smallnegy = -0.1
        p1 = normsubpath([normline_pt(-1, 0, 1, 0)])
        p2 = normsubpath([normline_pt(0, smallposy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, 1+smallnegy),
                          normline_pt(0, 1+smallnegy, 0, smallnegy),
                          normline_pt(0, smallnegy, 0, smallposy)], closed=0)
        p1.epsilon = p2.epsilon = 0.05
        intersect = p2.intersect(p1)
        self.assertEqual(len(intersect[0]), 4)
        self.assertAlmostEqual(intersect[0][0], 0.5)
        self.assertAlmostEqual(intersect[0][1], 1.1)
        self.assertAlmostEqual(intersect[0][2], 2.9)
        self.assertAlmostEqual(intersect[0][3], 3.5)


if __name__ == "__main__":
    unittest.main()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

from pyx import *

def isEqual(trafo1, trafo2):
    return max(list(map(abs,[trafo1.matrix[0][0]-trafo2.matrix[0][0],
                        trafo1.matrix[0][1]-trafo2.matrix[0][1],
                        trafo1.matrix[1][0]-trafo2.matrix[1][0],
                        trafo1.matrix[1][1]-trafo2.matrix[1][1],
                        trafo1.vector[0]-trafo2.vector[0],
                        trafo1.vector[1]-trafo2.vector[1]])))<1e-7

def correctOnBasis(t, tesx, tesy):
    esx = t.apply(1 * unit.t_cm, 0)
    esy = t.apply(0, 1 * unit.t_cm)

    esx = unit.tocm(esx[0]), unit.tocm(esx[1])
    esy = unit.tocm(esy[0]), unit.tocm(esy[1])

#    print "  (1,0) => (%f, %f)" % (esx[0], esx[1])
#    print "  (0,1) => (%f, %f)" % (esy[0], esy[1])

    return max(list(map(abs,[esx[0]-tesx[0], esx[1]-tesx[1],
                        esy[0]-tesy[0], esy[1]-tesy[1]])))<1e-7


class TrafoTestCase(unittest.TestCase):

    def testInverse(self):
        "t*t.inverse()=1"
        t = trafo.translate(-1,-1)*trafo.rotate(72)*trafo.translate(1,1)
        assert isEqual(t*t.inverse(), trafo.trafo()), \
                "wrong inverse definition"

    def testTranslate(self):
        assert correctOnBasis(trafo.translate(1,0),
                              (2,0), (1,1)), \
               "wrong definition of trafo.translation"
        assert correctOnBasis(trafo.translate(0,1),
                              (1,1), (0,2)), \
               "wrong definition of trafo.translation"

    def testRotate(self):
        assert correctOnBasis(trafo.rotate(90),
                              (0, 1), (-1, 0)), \
               "wrong definition of trafo.rotation"
        assert isEqual(trafo.rotate(360), trafo.trafo()), \
                "trafo.rotation by 360 deg is not 1"
        assert isEqual(trafo.rotate(40)*trafo.rotate(120)*
                       trafo.rotate(90)*trafo.rotate(110),
                        trafo.trafo()), \
                "successive multiplication by 360 degrees does not yield 1"

    def testMirror(self):
        "trafo.mirroring two times must yield 1 and -mirror(phi)=mirror(phi+180)"
        assert isEqual(trafo.mirror(20)*trafo.mirror(20),
                       trafo.trafo()), \
                "trafo.mirroring not idempotent"
        assert isEqual(trafo.mirror(20),
                       trafo.mirror(180+20)), \
                "trafo.mirroring by 20 degrees unequal to trafo.mirroring by 180+20 degrees"

    def testScale(self):
        assert correctOnBasis(trafo.scale(0.5),
                              (0.5,0), (0, 0.5)), \
               "wrong definition of trafo.scaling"
        assert correctOnBasis(trafo.scale(0.5, 0.2),
                              (0.5,0), (0, 0.2)), \
               "wrong definition of trafo.scaling"
        assert isEqual(trafo.scale(2,3)*trafo.scale(1/2.0, 1/3.0),
                       trafo.trafo()), \
                "trafo.scaling definition wrong"

    def testSlant(self):
        assert correctOnBasis(trafo.slant(0.5),
                              (1,0), (0.5, 1)), \
               "wrong definition of trafo.slant"
        assert isEqual(trafo.slant(2)*trafo.slant(-2),
                       trafo.trafo()), \
                "trafo.slant definition wrong"

    def testMultVsMethods(self):
        "test multiplication vs trafo methods"
        assert isEqual(trafo.rotate(72).translated(1,2),
                       trafo.translate(1,2)*trafo.rotate(72)), \
               "trafo.translate not consistent with multiplication result"
        assert isEqual(trafo.mirror(20)*trafo.mirror(20),
                       trafo.mirror(20).mirrored(20)), \
               "trafo.mirror not consistent with multiplication result"
        assert isEqual(trafo.translate(1,2).rotated(72).translated(-3,-1),
                       trafo.translate(-3,-1)*
                       trafo.rotate(72)*
                       trafo.translate(1,2)), \
                "trafo.translate/rotate not consistent with multiplication result"

    def testTranslateCombined(self):
        assert correctOnBasis(trafo.translate(1,0)*trafo.rotate(90),
                              (1,1), (0,0)), \
               "wrong trafo.translation/trafo.rotation definition"
        assert correctOnBasis(trafo.rotate(90)*trafo.translate(1,0),
                              (0,2), (-1,1)), \
               "wrong trafo.translation/trafo.rotation definition"
        assert isEqual(trafo.rotate(72,1,2),
                       trafo.translate(-1,-2).rotated(72).translated(1,2)), \
               "wrong translate/rotate definition"

        assert correctOnBasis(trafo.translate(1,0)*
                              trafo.scale(0.5),
                              (1.5, 0), (1.0,0.5)), \
               "wrong trafo.translation/trafo.scaling definition"
        assert correctOnBasis(trafo.scale(0.5)*
                              trafo.translate(1,0),
                              (1, 0), (0.5,0.5)), \
               "wrong trafo.translation/trafo.scaling definition"

        assert correctOnBasis(trafo.translate(1,0)*
                              trafo.rotate(90)*
                              trafo.scale(0.5),
                              (1,0.5), (0.5,0)), \
               "wrong trafo.translation/trafo.rotation/trafo.scaling definition"
        assert correctOnBasis(trafo.translate(1,0)*
                              trafo.scale(0.5)*
                              trafo.rotate(90),
                              (1,0.5), (0.5,0)), \
               "wrong trafo.translation/trafo.rotation/trafo.scaling definition"
        assert correctOnBasis(trafo.rotate(90)*trafo.scale(0.5)*trafo.translate(1,0),
                              (0,1), (-0.5,0.5)), \
               "wrong trafo.translation/trafo.rotation/trafo.scaling definition"


if __name__ == "__main__":
    unittest.main()



from pyx import *

c = canvas.canvas()

circle = path.circle(0, 0, 2)
line = path.line(-3, 1, 3, 2)
c.stroke(circle, [style.linewidth.Thick])
c.stroke(line, [style.linewidth.Thick])

isects_circle, isects_line = circle.intersect(line)
for isect in isects_circle:
    isectx, isecty = circle.at(isect)
    c.stroke(path.line(0, 0, isectx, isecty))

c.writePDFfile()



from math import pi
from pyx import *
from pyx.graph.axis import linear
from pyx.graph.axis.texter import rational

g = graph.graphxy(width=8, key=graph.key.key(pos="bl"),
                  x=linear(min=0, max=2*pi, title="$x$", divisor=pi,
                           texter=rational(suffix=r"\pi")),
                  y=linear(title="$y$"))

g.plot(graph.data.function("y(x)=sin(x)", title=r"$\sin(x)$"))
g.plot(graph.data.function("y(x)=cos(x)", title=r"$\cos(x)$"))

g.writeEPSfile("piaxis")



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *
from pyx.path import *

c=canvas.canvas()

unit.set(uscale=1)
c.stroke(path(moveto(0,0), lineto(unit.u_cm+unit.t_cm, unit.u_cm)), [color.rgb.red])
unit.set(uscale=2)
c.stroke(path(moveto(0,0), lineto(unit.u_cm+unit.t_cm, unit.u_cm)), [color.rgb.green])
unit.set(uscale=4)
c.stroke(path(moveto(0,0), lineto(unit.u_cm+unit.t_cm, unit.u_cm)), [color.rgb.blue])

c.writeEPSfile("test_unit")
c.writePDFfile("test_unit")
c.writeSVGfile("test_unit")




# contributed by Francisco Borges

from pyx import *
from pyx.graph import graphxy, data, axis
from pyx.graph.axis import painter, tick
from pyx.deco import earrow

range = 2.5

p = painter.regular(basepathattrs=[earrow.normal], titlepos=0.95,
                    outerticklength=painter.ticklength.normal,
                    titledist=-0.3, titledirection=None) # horizontal text

g = graphxy(width=10, xaxisat=0, yaxisat=0,
            x=axis.linear(title="$x$", min=-range, max=+range, painter=p,
                          # suppress some ticks by overwriting ...
                          manualticks=[tick.tick(0, None, None),
                                       tick.tick(range, None, None)]),
            y=axis.linear(title=r"$x\sin(x^2)$", painter=p,
                          manualticks=[tick.tick(0, None, None),
                                       tick.tick(3, None, None)]))

g.plot(data.function("y(x)=x*sin(x**2)"),
       # Line style is set before symbol style -> symbols will be draw
       # above the line.
       [graph.style.line([style.linewidth.Thin, style.linestyle.solid]),
        graph.style.symbol(graph.style.symbol.circle, size=0.1,
                           symbolattrs=[deco.filled([color.rgb.green]),
                                        deco.stroked([color.rgb.red])])])

# manually typeset "0" near the origin
g.dolayout()
x0, y0 = g.pos(0, 0)
g.text(x0 - 0.2, y0 - 0.2, "0", [text.halign.right, text.valign.top])

g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



#!/usr/bin/env python
import sys; sys.path.insert(0, "../..")

import math
from random import Random

from pyx import *
from pyx import normpath
from pyx.deformer import controldists_from_endgeometry_pt
from pyx.normpath import _epsilon

def test_reproductions(seed, n_tests, xmax, ymax, accuracy): # <<<
    # insert xmax, ymax, accuracy in pt!

    r = Random(seed)
    n_testpoints = 20
    testparams = [i / (n_testpoints - 1.0) for i in range(n_testpoints)]

    # a canvas for drawing
    can = canvas.canvas()
    xpos = 0
    ypos = 0

    # loop over many different cases
    n_failures = 0
    n_successes = 0
    for cnt in range(n_tests):

        # the original curve
        a = r.uniform(0,xmax), r.uniform(0,ymax)
        b = r.uniform(0,xmax), r.uniform(0,ymax)
        c = r.uniform(0,xmax), r.uniform(0,ymax)
        d = r.uniform(0,xmax), r.uniform(0,ymax)
        origcurve = normpath.normcurve_pt(a[0], a[1], b[0], b[1], c[0], c[1], d[0], d[1])
        tbeg, tend = origcurve.rotation([0, 1])
        cbeg, cend = origcurve.curvature_pt([0, 1])
        # raise an error if one of the params is invalid:
        tbeg, tend, cbeg, cend
        tbeg, tend = tbeg.apply_pt(1, 0), tend.apply_pt(1, 0)

        # the reproduced curve
        controldistpairs = controldists_from_endgeometry_pt(a, d, tbeg, tend, cbeg, cend, _epsilon)
        reprocurves = []
        for controldistpair in controldistpairs:
            alpha, beta = controldistpair
            reprocurves.append(normpath.normcurve_pt(
              a[0], a[1],
              a[0] + alpha * tbeg[0], a[1] + alpha * tbeg[1],
              d[0] - beta * tend[0], d[1] - beta * tend[1],
              d[0], d[1]))

        # analyse the quality of the reproduction
        minmaxdist = float("inf")
        minindex = -1
        maxdists = []
        for i, reprocurve in enumerate(reprocurves):
            maxdist = max([math.hypot(p[0]-q[0], p[1]-q[1])
              for p,q in zip(origcurve.at_pt(testparams), reprocurve.at_pt(testparams))])
            if maxdist < minmaxdist:
                minmaxdist = maxdist
                minindex = i
            maxdists.append(maxdist)

        # print complaints for too bad reproductions
        if minindex != 0 or minmaxdist > accuracy:
            n_failures += 1

            if minmaxdist > accuracy:
                print("%4d Smallest distance is %f" % (cnt, minmaxdist))

            if minindex == -1:
                print("%4d Failure: no solution found" % (cnt))

            if minindex > 0:
                print("%4d Wrong sorting: entry %d is the smallest" % (cnt, minindex))

            if minindex >=0 and not (controldistpairs[minindex][0] >= 0 and controldistpairs[minindex][1] >= 0):
                print("%4d Failure: signs are wrong" % (cnt))

            # selectively draw the curves:
            #if minindex == -1:
            #if minindex > 0:
            if minmaxdist > accuracy:
                # draw the failure curves
                can.stroke(path.rect_pt(0,0,xmax,ymax), [trafo.translate_pt(xpos, ypos)])
                can.draw(normpath.normpath([normpath.normsubpath([origcurve])]),  [trafo.translate_pt(xpos, ypos), deco.stroked([style.linewidth.THIck]), deco.shownormpath()])
                if minindex != -1:
                    can.stroke(normpath.normpath([normpath.normsubpath([reprocurves[minindex]])]), [trafo.translate_pt(xpos, ypos), color.rgb.red])
                can.text(0, 0, r"(%d)" % (cnt), [trafo.translate_pt(xpos+0.5*xmax, ypos), text.halign.center, text.vshift(2.0)])
                xpos += 1.1*xmax
                if xpos > 11*xmax:
                    xpos = 0
                    ypos -= 1.1*ymax
                    ypos -= 15
        else:
            n_successes += 1

    print("failures, successes = ", n_failures, ", ", n_successes)
    can.writeEPSfile("test_bezier")
# >>>

test_reproductions(43, 10000, 100, 100, 1.0e-2)


# vim:foldmethod=marker:foldmarker=<<<,>>>



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

from pyx import *
from pyx.deco import *

class DecoTestCase(unittest.TestCase):

    def testExcluderange(self):
        d = decoratedpath(path.line(0, 0, 1, 0))
        d.ensurenormpath()
        d.excluderange(0.1, 0.2)
        d.excluderange(0.5, 0.6)
        d.excluderange(0.3, 0.4)
        self.assertEqual(len(d.nostrokeranges), 3)
        self.assertAlmostEqual(d.nostrokeranges[0][0], 0.1)
        self.assertAlmostEqual(d.nostrokeranges[0][1], 0.2)
        self.assertAlmostEqual(d.nostrokeranges[1][0], 0.3)
        self.assertAlmostEqual(d.nostrokeranges[1][1], 0.4)
        self.assertAlmostEqual(d.nostrokeranges[2][0], 0.5)
        self.assertAlmostEqual(d.nostrokeranges[2][1], 0.6)

        d.excluderange(0.52, 0.58)
        self.assertEqual(len(d.nostrokeranges), 3)
        self.assertAlmostEqual(d.nostrokeranges[0][0], 0.1)
        self.assertAlmostEqual(d.nostrokeranges[0][1], 0.2)
        self.assertAlmostEqual(d.nostrokeranges[1][0], 0.3)
        self.assertAlmostEqual(d.nostrokeranges[1][1], 0.4)
        self.assertAlmostEqual(d.nostrokeranges[2][0], 0.5)
        self.assertAlmostEqual(d.nostrokeranges[2][1], 0.6)

        d.excluderange(0.45, 0.55)
        self.assertEqual(len(d.nostrokeranges), 3)
        self.assertAlmostEqual(d.nostrokeranges[0][0], 0.1)
        self.assertAlmostEqual(d.nostrokeranges[0][1], 0.2)
        self.assertAlmostEqual(d.nostrokeranges[1][0], 0.3)
        self.assertAlmostEqual(d.nostrokeranges[1][1], 0.4)
        self.assertAlmostEqual(d.nostrokeranges[2][0], 0.45)
        self.assertAlmostEqual(d.nostrokeranges[2][1], 0.6)

        d.excluderange(0.15, 0.25)
        self.assertEqual(len(d.nostrokeranges), 3)
        self.assertAlmostEqual(d.nostrokeranges[0][0], 0.1)
        self.assertAlmostEqual(d.nostrokeranges[0][1], 0.25)
        self.assertAlmostEqual(d.nostrokeranges[1][0], 0.3)
        self.assertAlmostEqual(d.nostrokeranges[1][1], 0.4)
        self.assertAlmostEqual(d.nostrokeranges[2][0], 0.45)
        self.assertAlmostEqual(d.nostrokeranges[2][1], 0.6)

        d.excluderange(0.05, 0.15)
        self.assertEqual(len(d.nostrokeranges), 3)
        self.assertAlmostEqual(d.nostrokeranges[0][0], 0.05)
        self.assertAlmostEqual(d.nostrokeranges[0][1], 0.25)
        self.assertAlmostEqual(d.nostrokeranges[1][0], 0.3)
        self.assertAlmostEqual(d.nostrokeranges[1][1], 0.4)
        self.assertAlmostEqual(d.nostrokeranges[2][0], 0.45)
        self.assertAlmostEqual(d.nostrokeranges[2][1], 0.6)

        d.excluderange(0.2, 0.35)
        self.assertEqual(len(d.nostrokeranges), 2)
        self.assertAlmostEqual(d.nostrokeranges[0][0], 0.05)
        self.assertAlmostEqual(d.nostrokeranges[0][1], 0.4)
        self.assertAlmostEqual(d.nostrokeranges[1][0], 0.45)
        self.assertAlmostEqual(d.nostrokeranges[1][1], 0.6)

        d.excluderange(0.35, 0.65)
        self.assertEqual(len(d.nostrokeranges), 1)
        self.assertAlmostEqual(d.nostrokeranges[0][0], 0.05)
        self.assertAlmostEqual(d.nostrokeranges[0][1], 0.65)


if __name__ == "__main__":
    unittest.main()



import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

import types
from pyx import *

class UnitTestCase(unittest.TestCase):

    def testTrueUnits(self):
        self.assertAlmostEqual(unit.tom(unit.t_m*42), 42)
        self.assertAlmostEqual(unit.tocm(unit.t_cm*42), 42)
        self.assertAlmostEqual(unit.tomm(unit.t_mm*42), 42)
        self.assertAlmostEqual(unit.toinch(unit.t_inch*42), 42)
        self.assertAlmostEqual(unit.topt(unit.t_pt*42), 42)

    def testUserUnits(self):
        unit.set(uscale=2)
        self.assertAlmostEqual(unit.tom(unit.m*42), 84)
        self.assertAlmostEqual(unit.tocm(unit.cm*42), 84)
        self.assertAlmostEqual(unit.tomm(unit.mm*42), 84)
        self.assertAlmostEqual(unit.toinch(unit.inch*42), 84)
        self.assertAlmostEqual(unit.topt(unit.pt*42), 84)
        unit.set(uscale=1)

    def testVisualUnits(self):
        unit.set(vscale=3)
        self.assertAlmostEqual(unit.tom(unit.v_m*42), 126)
        self.assertAlmostEqual(unit.tocm(unit.v_cm*42), 126)
        self.assertAlmostEqual(unit.tomm(unit.v_mm*42), 126)
        self.assertAlmostEqual(unit.toinch(unit.v_inch*42), 126)
        self.assertAlmostEqual(unit.topt(unit.v_pt*42), 126)
        unit.set(vscale=1)

    def testWidthUnits(self):
        unit.set(wscale=4)
        self.assertAlmostEqual(unit.tom(unit.w_m*42), 168)
        self.assertAlmostEqual(unit.tocm(unit.w_cm*42), 168)
        self.assertAlmostEqual(unit.tomm(unit.w_mm*42), 168)
        self.assertAlmostEqual(unit.toinch(unit.w_inch*42), 168)
        self.assertAlmostEqual(unit.topt(unit.w_pt*42), 168)
        unit.set(wscale=1)

    def testTeXUnits(self):
        unit.set(xscale=5)
        self.assertAlmostEqual(unit.tom(unit.x_m*42), 210)
        self.assertAlmostEqual(unit.tocm(unit.x_cm*42), 210)
        self.assertAlmostEqual(unit.tomm(unit.x_mm*42), 210)
        self.assertAlmostEqual(unit.toinch(unit.x_inch*42), 210)
        self.assertAlmostEqual(unit.topt(unit.x_pt*42), 210)
        unit.set(xscale=1)

    def testMixedUnits(self):
        unit.set(uscale=2, vscale=3, wscale=4, xscale=5)
        self.assertAlmostEqual(unit.tom(42*unit.t_m + 42*unit.m + 42*unit.v_m + 42*unit.w_m + 42*unit.x_m), 630)
        self.assertAlmostEqual(unit.tocm(42*unit.t_cm + 42*unit.cm + 42*unit.v_cm + 42*unit.w_cm + 42*unit.x_cm), 630)
        self.assertAlmostEqual(unit.toinch(42*unit.t_inch + 42*unit.inch + 42*unit.v_inch + 42*unit.w_inch + 42*unit.x_inch), 630)
        self.assertAlmostEqual(unit.topt(42*unit.t_pt + 42*unit.pt + 42*unit.v_pt + 42*unit.w_pt + 42*unit.x_pt), 630)
        unit.set(uscale=1, vscale=1, wscale=1, xscale=1)

    def testCompareUnits(self):
        assert unit.cm*41 < unit.cm*42, "comparision error"
        assert unit.cm*42 == unit.cm*42, "numbers should be equal (might be unstable)"
        assert unit.cm*43 > unit.cm*42, "comparision error"
        assert unit.cm*41 < 42, "comparision error"
        assert unit.cm*42 == 42, "numbers should be equal (might be unstable)"
        assert unit.cm*43 > 42, "comparision error"
        assert 42 > unit.cm*41, "comparision error"
        assert 42 == unit.cm*42, "numbers should be equal (might be unstable)"
        assert 42 < unit.cm*43, "comparision error"


if __name__ == "__main__":
    unittest.main()



import base64, os, re, sys
import nbformat.v4 as nbf

filename = os.path.splitext(sys.argv[1])[0]

try:
    title, description = open("{}.txt".format(filename), encoding="utf-8").read().split('\n\n', 1)
except IOError:
    title, description = filename, ""
description = description.replace("...", "").replace("'''", "**").replace("''", "*")
bendpattern = re.compile("^!+", re.MULTILINE)
bendcode = '<img src="http://pyx.sourceforge.net/bend.png" align="left">'
description = re.sub(bendpattern, lambda m: bendcode*(m.end()-m.start()), description)
code = open("{}.py".format(filename), encoding="utf-8").read()
code = re.sub(r'\.writeEPSfile\(("[a-z]+")?\)\n.*writePDFfile\(("[a-z]+")?\)\n.*writeSVGfile\(("[a-z]+")?\)\n', "", code)

nb = nbf.new_notebook()
cells = []
cells.append(nbf.new_markdown_cell(source="# " + title))
cells.append(nbf.new_code_cell(source=code, execution_count=1,
                               outputs=[nbf.new_output(output_type=u'execute_result', execution_count=1,
                                                       data={'image/png': base64.encodebytes(open("{}.png".format(filename), "rb").read()).decode("ascii"),
                                                             'image/svg+xml': open("{}.svg".format(filename), "r", encoding="utf-8").read()})]))
cells.append(nbf.new_markdown_cell(source=description))
nb = nbf.new_notebook(cells=cells, metadata={'language': 'python'})
open("{}.ipynb".format(filename), "w").write(nbf.writes(nb))



# contributed by Sigmund Kohler

from math import pi, cos
from pyx import *
from pyx.deco import barrow, earrow
from pyx.style import linewidth, linestyle
from pyx.graph import graphxy
from pyx.graph.axis import linear
from pyx.graph.axis.painter import regular
from pyx.graph.style import line
from pyx.graph.data import function

mypainter = regular(basepathattrs=[earrow.normal], titlepos=1)
def mycos(x): return -cos(x)+.10*x

g = graphxy(height=5, x2=None, y2=None,
            x=linear(min=-2.5*pi, max=3.3*pi, parter=None,
                      painter=mypainter, title=r"$\delta\phi$"),
            y=linear(min=-2.3, max=2, painter=None))
g.plot(function("y(x)=mycos(x)", context=locals()),
       [line(lineattrs=[linewidth.Thick])])
g.finish()

x1, y1 = g.pos(-pi+.1, mycos(-pi+.1))
x2, y2 = g.pos(-.1, mycos(-.1))
x3, y3 = g.pos(pi+.1, mycos(pi+.1))

g.stroke(path.line(x1-.5, y1, x1+.5, y1), [linestyle.dashed])
g.stroke(path.line(x1-.5, y3, x3+.5, y3), [linestyle.dashed])
g.stroke(path.line(x2-.5, y2, x3+.5, y2), [linestyle.dashed])
g.stroke(path.line(x1, y1, x1, y3), [barrow.normal, earrow.normal])
g.stroke(path.line(x3, y2, x3, y3), [barrow.normal, earrow.normal])
g.text(x1+.2, 0.5*(y1+y3), r"$2\pi\gamma k\Omega$", [text.vshift.middlezero])
g.text(x1-.6, y1-.1, r"$E_{\rm b}$", [text.halign.right])
g.text(x3+.15, y2+.20, r"$2J_k(\varepsilon/\Omega)+\pi\gamma k\Omega$")

g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

p = lambda phi: graph.graphxyz.central(10, phi, 10-phi/18.0)
# p = lambda phi: graph.graphxyz.parallel(phi, 10/18.0)

d = document.document()
phi = 0.123
while phi < 360:
    c = canvas.canvas()
    g = c.insert(graph.graphxyz(10.5, 10, size=6, projector=p(phi),
                                x=graph.axis.lin(painter=graph.axis.painter.regular(gridattrs=[])),
                                y=graph.axis.lin(painter=graph.axis.painter.regular(gridattrs=[])),
                                z=graph.axis.lin(painter=graph.axis.painter.regular(gridattrs=[]))))
    g.plot(graph.data.values(x=[0, 0, 0, 1], y=[0, 0, 1, 1], z=[0, 1, 0, 1], color=[0, 0.33, 0.67, 1]),
           [graph.style.gridpos(index1=1, index2=2), graph.style.surface(gridcolor=color.gray(0.9))])
    g.doplot()
    g.text(2, 2, str(phi))
    def vrect(vx1, vy1, vz1, vx2, vy2, vz2):
        assert len([v1_v2 for v1_v2 in [(vx1, vx2), (vy1, vy2), (vz1, vz2)] if v1_v2[0] == v1_v2[1]]) == 1
        if vz1 == vz2:
            return path.path(path.moveto_pt(*g.vpos_pt(vx1, vy1, vz1)),
                             path.lineto_pt(*g.vpos_pt(vx1, vy2, vz1)),
                             path.lineto_pt(*g.vpos_pt(vx2, vy2, vz1)),
                             path.lineto_pt(*g.vpos_pt(vx2, vy1, vz1)),
                             path.closepath())
        if vy1 == vy2:
            return path.path(path.moveto_pt(*g.vpos_pt(vx1, vy1, vz1)),
                             path.lineto_pt(*g.vpos_pt(vx1, vy1, vz2)),
                             path.lineto_pt(*g.vpos_pt(vx2, vy1, vz2)),
                             path.lineto_pt(*g.vpos_pt(vx2, vy1, vz1)),
                             path.closepath())
        if vx1 == vx2:
            return path.path(path.moveto_pt(*g.vpos_pt(vx1, vy1, vz1)),
                             path.lineto_pt(*g.vpos_pt(vx1, vy2, vz1)),
                             path.lineto_pt(*g.vpos_pt(vx1, vy2, vz2)),
                             path.lineto_pt(*g.vpos_pt(vx1, vy1, vz2)),
                             path.closepath())
    g.fill(vrect(0.4, 0.4, 0, 0.6, 0.6, 0), [g.pz0show and color.rgb.green or color.rgb.red])
    g.fill(vrect(0.4, 0.4, 1, 0.6, 0.6, 1), [g.pz1show and color.rgb.green or color.rgb.red])
    g.fill(vrect(0.4, 0, 0.4, 0.6, 0, 0.6), [g.py0show and color.rgb.green or color.rgb.red])
    g.fill(vrect(0.4, 1, 0.4, 0.6, 1, 0.6), [g.py1show and color.rgb.green or color.rgb.red])
    g.fill(vrect(0, 0.4, 0.4, 0, 0.6, 0.6), [g.px0show and color.rgb.green or color.rgb.red])
    g.fill(vrect(1, 0.4, 0.4, 1, 0.6, 0.6), [g.px1show and color.rgb.green or color.rgb.red])
    for axis in list(g.axes.values()):
        g.stroke(axis.basepath(), [axis.hidden and color.rgb.red or color.rgb.green])
    d.append(document.page(c))
    phi += 5.67

d.writePSfile("test_graph3dloop")
d.writePDFfile("test_graph3dloop")




import sys; sys.path.insert(0, "../..")
from pyx import *
try:
    from PIL.Image import open
except ImportError:
    print("PIL not available, skipping palette test")
    paletteimage = None
else:
    paletteimage = open("data/vcss.png")

image_bw = bitmap.image(2, 2, "L", b"\0\377\377\0")
image_rgb = bitmap.image(3, 2, "RGB", b"\77\77\77\177\177\177\277\277\277"
                                      b"\377\0\0\0\377\0\0\0\377")

bitmap_bw_stream = bitmap.bitmap(0, 1, image_bw, height=0.8)
bitmap_rgb_stream = bitmap.bitmap(0, 0, image_rgb, height=0.8)

bitmap_bw_storestring = bitmap.bitmap(2, 1, image_bw, height=0.8, PSstoreimage=1)
bitmap_rgb_storestring = bitmap.bitmap(2, 0, image_rgb, height=0.8, PSstoreimage=1)

bitmap_bw_storearray = bitmap.bitmap(4, 1, image_bw, height=0.8, PSstoreimage=1, PSmaxstrlen=2)
bitmap_rgb_storearray = bitmap.bitmap(4, 0, image_rgb, height=0.8, PSstoreimage=1, PSmaxstrlen=2)

c = canvas.canvas()
c.insert(bitmap_bw_stream)
c.insert(bitmap_rgb_stream)
c.insert(bitmap_bw_storestring)
c.insert(bitmap_rgb_storestring)
c.insert(bitmap_bw_storearray)
c.insert(bitmap_rgb_storearray)
if paletteimage:
    c.insert(bitmap.bitmap(6, 0, paletteimage, height=1.8))
c.writeEPSfile("test_bitmap", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_bitmap", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_bitmap", page_paperformat=document.paperformat.A4)




from pyx import *

c = canvas.canvas()

p = path.path(path.moveto(0, 0), path.arc(0, 0, 1, 210, 150), path.closepath())
c.stroke(p, [style.linewidth(0.4)])

pp = deformer.parallel(-0.2, sharpoutercorners=1).deform(p) + deformer.parallel( 0.2, sharpoutercorners=1).deform(p).reversed()
c.stroke(pp, [trafo.translate(2.5, 0), deco.filled([color.rgb.red])])

# PyX gets the bounding box wrong, which leads to the clipping on the left
c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



from pyx import *
from pyx.connector import arc, curve

unit.set(uscale=3)

c = canvas.canvas()

textattrs = [text.halign.center, text.vshift.middlezero]
A = text.text(0, 0, r"\bf A", textattrs)
B = text.text(1, 0, r"\bf B", textattrs)
C = text.text(1, 1, r"\bf C", textattrs)
D = text.text(0, 1, r"\bf D", textattrs)

for X in [A, B, C, D]:
    c.draw(X.bbox().enlarged(0.1).path(),
        [deco.stroked(), deco.filled([color.grey(0.85)])])
    c.insert(X)

for X,Y in [[A, B], [B, C], [C, D], [D, A]]:
    c.stroke(arc(X, Y, boxdists=[0.2, 0.2], relangle=-45), [color.rgb.red, deco.earrow.normal])

c.stroke(curve(D, B, boxdists=[0.2, 0.2], relangle1=-45, relangle2=-45, relbulge=0.8),
         [color.rgb.blue, deco.earrow.normal])

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



import math
from pyx import *

c = canvas.canvas()
t1 = text.text(0, 0, "circle align", [text.halign.center, text.size.huge])
x, y = t1.circlealignvector(1, 1, 1)
t1.circlealign(1, 1, 1)
c.stroke(path.path(path.arc(0, 0, 1, -30, 120)), [style.linestyle.dashed])
c.stroke(t1.path(centerradius=0.05), [style.linewidth.THin])
c.stroke(path.line(0, 0, x, y), [deco.earrow.normal])
c.insert(t1)
t2 = text.text(5, 0, "line align", [text.halign.center, text.size.huge])
x, y = t2.linealignvector(1, 1, 1)
t2.linealign(1, 1, 1)
c.stroke(path.line(5 + 1.5 * math.sqrt(2), -0.5 * math.sqrt(2), 5 - 0.5 * math.sqrt(2), 1.5 * math.sqrt(2)), [style.linestyle.dashed])
c.stroke(t2.path(centerradius=0.05), [style.linewidth.THin])
c.stroke(path.line(5, 0, 5 + x, y), [deco.earrow.normal])
c.insert(t2)
c.writePDFfile()




#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

import time, datetime
from pyx import *
from pyx.graph.axis import timeaxis
from pyx.graph import data

d = data.file("data/timedata", date=1, value=2)
d = data.points([[datetime.datetime(*(time.strptime(date)[:6])), value] for date, value in zip(d.columns["date"], d.columns["value"])], x=1, y=2)

g = graph.graphxy(height=5, x=timeaxis.timeaxis(manualticks=[timeaxis.timetick(2003, 8, 12),
                                                             timeaxis.timetick(2003, 8, 13),
                                                             timeaxis.timetick(2003, 8, 14),
                                                             timeaxis.timetick(2003, 8, 15),
                                                             timeaxis.timetick(2003, 8, 16)],
                                                texter=timeaxis.timetexter("%d %b")))
g.plot(d)
g.writeEPSfile("test_timeaxis")
g.writePDFfile("test_timeaxis")
g.writeSVGfile("test_timeaxis")




#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

import pyx
from pyx import *
from pyx.path import *
import math

def testarrow(c):
    c.stroke(path(moveto(10,20),
                curveto(12,16,14,15,12,19),
                rcurveto(-3,2,3,3,-2,4)),
             [deco.barrow.small, deco.earrow.normal,
              deco.arrow(pos=0.5),
              deco.arrow(pos=0.7, reversed=1)])

    c.stroke(path(arc(8,15,4,10,70)), [deco.barrow.small, deco.earrow.normal])
    c.stroke(path(arc(8,15,3,10,70)), [deco.barrow.small, deco.earrow.normal])
    c.stroke(path(arc(8,15,2,10,70)), [deco.barrow.small, deco.earrow.normal])
    c.stroke(path(arc(8,15,1,10,70)), [deco.barrow.small, deco.earrow.normal])
    c.stroke(path(arc(8,15,0.5,10,70)), [deco.barrow.small, deco.earrow.normal])

    base = 2

    pal = color.lineargradient_rgb(color.rgb.red, color.rgb.blue)

    c.stroke(path(moveto(5,10), rlineto(5,0)),
           [#deco.barrow(size=base/math.sqrt(8)*unit.t_pt, constriction=1),
            deco.earrow.SMall,
            deco.colorgradient(pal),
            deco.text("start", arclenfrombegin=0, angle=90)])
    c.stroke(path(moveto(5,10.5), rlineto(5,0)),
           [deco.barrow(size=base/math.sqrt(4)*unit.t_pt, constriction=1),
            deco.earrow.Small,
            deco.colorgradient(pal),
            deco.text("start+1", arclenfrombegin=1, angle=90)])
    c.stroke(path(moveto(5,11), rlineto(5,0)),
           [deco.barrow(size=base/math.sqrt(2)*unit.t_pt, constriction=1),
            deco.earrow.small,
            deco.colorgradient(pal),
            deco.text("center", angle=90)])
    c.stroke(path(moveto(5,11.5), rlineto(5,0)),
           [deco.barrow(size=base/math.sqrt(1)*unit.t_pt, constriction=1),
            deco.earrow.normal,
            deco.colorgradient(pal),
            deco.text("end-1", arclenfromend=1, angle=90)])
    c.stroke(path(moveto(5,12), rlineto(5,0)),
           [deco.barrow(size=base*math.sqrt(2)*unit.t_pt, constriction=1),
            deco.earrow.large,
            deco.text("end", arclenfromend=0, angle=90)])
    c.stroke(path(moveto(5,12.5), rlineto(5,0)),
           [deco.barrow(size=base*math.sqrt(4)*unit.t_pt, constriction=1),
            deco.earrow.Large])
    c.stroke(path(moveto(5,13), rlineto(5,0)),
           [deco.barrow(size=base*math.sqrt(8)*unit.t_pt, constriction=1),
            deco.earrow.LArge])
    c.stroke(path(moveto(5,13.5), rlineto(5,0)),
           [deco.barrow(size=base*math.sqrt(16)*unit.t_pt, constriction=1),
            deco.earrow.LARge])

    lt = style.linewidth.THick

    c.stroke(path(moveto(11,10), rlineto(5,0)),
           [lt,
            deco.barrow(size=base/math.sqrt(8)*unit.t_pt, constriction=1),
            deco.earrow.SMall,
            deco.colorgradient(pal, [style.linewidth.THIN])])
    c.stroke(path(moveto(11,10.5), rlineto(5,0)),
           [lt,
            deco.barrow(size=base/math.sqrt(4)*unit.t_pt, constriction=1),
            deco.earrow.Small])
    c.stroke(path(moveto(11,11), rlineto(5,0)),
           [lt,
            deco.barrow(size=base/math.sqrt(2)*unit.t_pt, constriction=1),
            deco.earrow.small])
    c.stroke(path(moveto(11,11.5), rlineto(5,0)),
           [lt,
            deco.barrow(size=base/math.sqrt(1)*unit.t_pt, constriction=1),
            deco.earrow.normal])
    c.stroke(path(moveto(11,12), rlineto(5,0)),
           [lt,
            deco.barrow(size=base*math.sqrt(2)*unit.t_pt, constriction=1),
            deco.earrow.large])
    c.stroke(path(moveto(11,12.5), rlineto(5,0)),
           [lt,
            deco.barrow(size=base*math.sqrt(4)*unit.t_pt, constriction=1),
            deco.earrow.Large(attrs=[deco.stroked([style.linestyle.dashed]), deco.filled.clear])])
    c.stroke(path(moveto(11,13), rlineto(5,0)),
           [lt,
            deco.barrow(size=base*math.sqrt(8)*unit.t_pt, constriction=1),
            deco.earrow.LArge(attrs=[style.linestyle.dashed, color.rgb.green])])
    c.stroke(path(moveto(11,13.5), rlineto(5,0)),
           [lt,
            deco.barrow(size=base*math.sqrt(16)*unit.t_pt, constriction=1),
            deco.earrow.LARge(attrs=[color.rgb.red,
                                     deco.stroked([style.linejoin.round]),
                                     deco.filled([color.rgb.blue])])])

def linehatched(c):
    p = rect(0, 0, 5, 0.5).transformed(trafo.rotate(60))
    c.draw(p, [deco.linehatched45.normal])
    c.fill(p, [trafo.translate(2,0), deco.linehatched(0.1, 60, cross=1)]) # should not be filled, but hatched
    c.stroke(p, [trafo.translate(4,0), deco.linehatched90.normal])
    c.draw(p, [trafo.translate(6,0), deco.linehatched(0.1, 30, strokestyles=[color.rgb.red, style.linewidth.normal], cross=1)])

def testbrace(c):
    p = line(0, 0, 5, 0).transformed(trafo.rotate(60))
    c.stroke(p, [deco.brace(dist=-0.1, stretch=1.1), deco.text(r"hallo", textdist=20*unit.x_pt, relangle=-90), trafo.translate(8,0)])

    tr = trafo.translate(12, 0)
    c.draw(line(0,0,0,4), [deco.leftbrace, tr])
    c.draw(line(4,0,4,4), [deco.rightbrace, tr])
    c.draw(line(0,0,4,0), [deco.belowbrace, tr])
    c.stroke(line(0,4,4,4), [deco.abovebrace, tr, style.linewidth.THin])

    tr = trafo.translate(12.5, 0.5)
    c.stroke(line(0,0,3,0), [deco.straightbrace, tr, color.rgb.red])
    c.draw(line(3,0,3,3), [deco.straightbrace(fillattrs=[color.rgb.blue]), tr])
    c.draw(line(3,3,0,3), [deco.straightbrace, tr])
    c.draw(line(0,3,0,0), [deco.straightbrace, tr])

c = canvas.canvas()
testarrow(c)
linehatched(c)
testbrace(c)
c.writeEPSfile("test_arrow", page_paperformat=document.paperformat.A4, page_rotated=0, page_fittosize=1)
c.writePDFfile("test_arrow", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_arrow", page_paperformat=document.paperformat.A4)




from pyx import *
unit.set(uscale=3, wscale=3)

dist = 0.15           # distance between the ropes
thick = 0.08          # thickness of the ropes

# build the basepath of the knot. This is the curve that lies
# between both ropes.
A = 1.0, 0.1    # point where the knot starts after the straight line
B = 1.3, 0      # point where the knot is curved, overlying the straight line
t = -0.8, -0.1  # direction at A
s =  0.3, 1.2   # direction at B
pts = [
( A[0]-t[0], A[1]-t[1]), ( A[0]     , A[1]     ),
  ( A[0]+t[0], A[1]+t[1]), (-B[0]-s[0],-B[1]-s[1]),
  (-B[0]     ,-B[1]     ), (-B[0]+s[0],-B[1]+s[1]),
  ( B[0]-s[0], B[1]-s[1]), ( B[0]     , B[1]     ),
  ( B[0]+s[0], B[1]+s[1]), (-A[0]-t[0],-A[1]-t[1]),
  (-A[0]     ,-A[1]     ), (-A[0]+t[0],-A[1]+t[1]) ]
seam = path.path(
  path.moveto(*(pts[0])),
  path.lineto(*(pts[1])),
  path.curveto(pts[2][0],pts[2][1],pts[3][0],pts[3][1],pts[4][0],pts[4][1]),
  path.curveto(pts[5][0],pts[5][1],pts[6][0],pts[6][1],pts[7][0],pts[7][1]),
  path.curveto(pts[8][0],pts[8][1],pts[9][0],pts[9][1],pts[10][0],pts[10][1]),
  path.lineto(*(pts[11])) )
# We smooth this curve a little because we used curveto together with lineto
seam = deformer.smoothed(0.6).deform(seam)

# The ropes, when drawn later, will have to overlap in a very specific way.
l = seam.arclen()
epsilon = 0.002*l # small overlap between the segments
seam_segs = seam.split([0.28*l+epsilon, 0.28*l-epsilon,
                        0.42*l+epsilon, 0.42*l-epsilon,
                        0.58*l+epsilon, 0.58*l-epsilon,
                        0.72*l+epsilon, 0.72*l-epsilon])[::2]

# For each segment, two shifted paths build the boundaries of each rope
ropes_segs = []
for seam_seg in seam_segs:
    ropes_segs.append([])
    for ropeshift in [-0.5*dist, 0.5*dist]:
        ropes_segs[-1].append([])
        for edgeshift in [-0.5*thick, 0.5*thick]:
            ropes_segs[-1][-1].append(
              deformer.parallel(ropeshift + edgeshift).deform(seam_seg))
# Now, ropes_segs is a list of segments, containing a list of ropes,
# each containing the two bounding paths of the rope segment

rope_colors = [color.rgb.blue, color.rgb.red]
c = canvas.canvas()
# Because the segments should overlap in a very specific manner
# we have to draw them in the correct order
for index in [1, 4, 2, 0, 3]:
    for rope_seg, col in zip(ropes_segs[index], rope_colors):
        seg = rope_seg[0].joined(rope_seg[1].reversed())
        seg.append(path.closepath())
        c.fill(seg, [col])
        c.stroke(rope_seg[0], [style.linecap.round, style.linejoin.round])
        c.stroke(rope_seg[1], [style.linecap.round, style.linejoin.round])

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
#
# Copyright (C) 2005-2011 André Wobst <wobsta@pyx-project.org>
#
# dviconvert is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# epstopng is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with epstopng; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from optparse import OptionParser
from pyx import *
from pyx import bbox, version
from pyx.dvi import dvifile

parser = OptionParser(usage="usage: %prog [-b] [-p paperformat] -o output-file dvi-file",
                      version="%prog " + version.version)
parser.add_option("-o", "--output",
                  type="string", dest="output",
                  help="output-file")
parser.add_option("-p", "--paperformat",
                  type="string", dest="paperformat", default=None,
                  help="optional paper format string")
parser.add_option("-b", "--writebbox",
                  action="store_true", dest="writebbox", default=0,
                  help="Add bouding box information on PS and PDF when a paperformat is defined")
(options, args) = parser.parse_args()
if len(args) != 1:
    parser.error("can process a single dvi-file only")

if options.paperformat:
    options.paperformat = getattr(document.paperformat, options.paperformat)
df = dvifile.DVIfile(args[0])
d = document.document()
while 1:
    dvipage = df.readpage()
    if not dvipage:
        break
    if options.paperformat:
        aligntrafo = trafo.translate(unit.t_inch, -unit.t_inch + options.paperformat.height)
        aligneddvipage = canvas.canvas([aligntrafo])
        aligneddvipage.insert(dvipage)
        p = document.page(aligneddvipage, paperformat=options.paperformat, centered=0)
    else:
        p = document.page(dvipage)
    d.append(p)
if options.writebbox:
    d.writetofile(options.output)
else:
    d.writetofile(options.output, writebbox=1)



from pyx import *

text.set(text.LatexRunner)
text.preamble(r"\renewcommand{\familydefault}{\ttdefault}")
c = canvas.canvas()

# positioning is quite ugly ... but it works at the moment
x = 0
y = 0
dx = 6
dy = -0.65
length = 0.8

def drawstyle(name, showpath=0, default=0):
    global x,y
    p = path.path(path.moveto(x + 0.1, y+0.1 ),
                       path.rlineto(length/2.0, 0.3),
                       path.rlineto(length/2.0, -0.3))
    c.stroke(p, [style.linewidth.THIck,  eval("style."+name)])
    if showpath:
        c.stroke(p, [style.linewidth.Thin, color.gray.white])
    if default:
        name = name + r"\rm\quad (default)"
    c.text(x + 1.5, y + 0.15, name, [text.size.footnotesize])
    y += dy
    if y < -16:
        y = 0
        x += dx


drawstyle("linecap.butt", showpath=1, default=1)
drawstyle("linecap.round", showpath=1)
drawstyle("linecap.square", showpath=1)

y += dy

drawstyle("linejoin.miter", showpath=1, default=1)
drawstyle("linejoin.round", showpath=1)
drawstyle("linejoin.bevel", showpath=1)

y += dy

drawstyle("linestyle.solid", default=1)
drawstyle("linestyle.dashed")
drawstyle("linestyle.dotted")
drawstyle("linestyle.dashdotted")

y += dy

drawstyle("linewidth.THIN")
drawstyle("linewidth.THIn")
drawstyle("linewidth.THin")
drawstyle("linewidth.Thin")
drawstyle("linewidth.thin")
drawstyle("linewidth.normal", default=1)
drawstyle("linewidth.thick")
drawstyle("linewidth.Thick")
drawstyle("linewidth.THick")
drawstyle("linewidth.THIck")
drawstyle("linewidth.THICk")
drawstyle("linewidth.THICK")

drawstyle("miterlimit.lessthan180deg", showpath=1)
drawstyle("miterlimit.lessthan90deg", showpath=1)
drawstyle("miterlimit.lessthan60deg", showpath=1)
drawstyle("miterlimit.lessthan45deg", showpath=1)
drawstyle("miterlimit.lessthan11deg", showpath=1, default=1)

y += dy

drawstyle("dash((1, 1, 2, 2, 3, 3), 0)")
drawstyle("dash((1, 1, 2, 2, 3, 3), 1)")
drawstyle("dash((1, 2, 3), 2)")
drawstyle("dash((1, 2, 3), 3)")
drawstyle("dash((1, 2, 3), 4)")
drawstyle("dash((1, 2, 3), rellengths=1)")


c.writePDFfile()



import numpy as np
from skimage.measure import find_contours
from pyx import color, graph

xmax = 1.6
xmin = -xmax
ymin = -1.4
ymax = 1.8
npts = 101
x, y = np.mgrid[xmin:xmax:npts*1j, ymin:ymax:npts*1j]
z = 0.5*(x*x+y*y)+(x*x*y-y**3/3)
data = list(zip(x.flat, y.flat, z.flat))

g = graph.graphxy(height=8, width=8,
                  x=graph.axis.linear(min=xmin, max=xmax, title="$x$"),
                  y=graph.axis.linear(min=ymin, max=ymax, title="$y$"))
g.plot(graph.data.points(data, x=1, y=2, color=3),
       [graph.style.density(gradient=color.gradient.Jet,
                            coloraxis=graph.axis.linear(min=-0.5, max=6,
                                                        title="$V$"))])

for contourlevel in np.concatenate((np.arange(-0.4, 0, 0.1),
                                    np.arange(0, 0.2, 0.05),
                                    np.arange(0.2, 6, 0.5)
                                   )):
    contours = find_contours(z, contourlevel)
    for contourdata in contours:
        contourdata_rescaled = [(xmin+(xmax-xmin)*x/(npts-1),
                                 ymin+(ymax-ymin)*y/(npts-1))
                                for x, y, in contourdata]
        g.plot(graph.data.points(contourdata_rescaled, x=1, y=2),
               [graph.style.line([color.rgb(1, 1, 1)])])
g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

import math
from pyx import *

text.set(text.LatexEngine)

def test_minimal(c, x, y):
    g = c.insert(graph.graphxyz(x, y, size=3))
    g.plot(graph.data.file("data/husimi_small.dat", x=1, y=2, z=3))

def test_line(c, x, y):
    g = c.insert(graph.graphxyz(x, y, size=3))
    g.plot(graph.data.file("data/husimi_small.dat", x=1, y=2, z=3), [graph.style.line()])

def test_grid(c, x, y):
    g = c.insert(graph.graphxyz(x, y, size=3, x=graph.axis.lin(title="x axis"), y=graph.axis.lin(title="y axis"), z=graph.axis.lin(title="z axis")))
    g.plot(graph.data.file("data/husimi_small.dat", x=1, y=2, z=3), [graph.style.grid()])

def test_surface(c, x, y):
    g = c.insert(graph.graphxyz(x, y, size=3, x=graph.axis.lin(), y=graph.axis.lin(), x2=None, y2=None))
    g.plot(graph.data.file("data/husimi_small.dat", x=1, y=2, z=3, color=4), [graph.style.surface()])

def test_surface2d(c, x, y):
    g = c.insert(graph.graphxy(x, y, width=6, height=6))
    g.plot(graph.data.file("data/husimi_small.dat", x=1, y=2, color=3), [graph.style.surface()])

c = canvas.canvas()
test_minimal(c, 0, 0)
test_line(c, 0, -8)
test_grid(c, 8, 0)
test_surface(c, 8, -8)
test_surface2d(c, -3, -20)

c.writeEPSfile("test_graph3d", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_graph3d", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_graph3d", page_paperformat=document.paperformat.A4)




import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

import math
from pyx import *

class BoxTestCase(unittest.TestCase):

    def testDistance(self):
        b1 = box.polygon(center=(0.5, math.sqrt(3)/6), corners=((0, 0), (1, 0), (0.5, math.sqrt(3)/2)))
        b2 = box.polygon(center=(0.5, math.sqrt(3)/6), corners=((0, 0), (1, 0), (0.5, math.sqrt(3)/2)))
        b2.transform(trafo.translate(3, 0))
        b3 = box.polygon(center=(0.5, math.sqrt(3)/6), corners=((0, 0), (1, 0), (0.5, math.sqrt(3)/2)))
        b3.transform(trafo.translate(3, 3 * math.tan(math.pi/6)))
        b4 = box.polygon(center=(0.5, math.sqrt(3)/6), corners=((0, 0), (1, 0), (0.5, math.sqrt(3)/2)))
        b4.transform(trafo.translate(0, 3))
        b5 = box.polygon(center=(0.5, math.sqrt(3)/6), corners=((0, 0), (1, 0), (0.5, math.sqrt(3)/2)))
        b5.transform(trafo.translate(0.5, 0.5))
        self.assertAlmostEqual(unit.topt(b1.boxdistance(b2)), unit.topt(2))
        self.assertAlmostEqual(unit.topt(b1.boxdistance(b3)), unit.topt(math.sqrt(9*(1 + math.tan(math.pi/6)**2)) - math.sqrt(3)/2))
        self.assertAlmostEqual(unit.topt(b1.boxdistance(b4)), unit.topt(3 - math.sqrt(3)/2))
        self.assertAlmostEqual(unit.topt(b2.boxdistance(b1)), unit.topt(2))
        self.assertAlmostEqual(unit.topt(b3.boxdistance(b1)), unit.topt(math.sqrt(9*(1 + math.tan(math.pi/6)**2)) - math.sqrt(3)/2))
        self.assertAlmostEqual(unit.topt(b4.boxdistance(b1)), unit.topt(3 - math.sqrt(3)/2))
        self.assertRaises(box.BoxCrossError, b1.boxdistance, b5)
        self.assertRaises(box.BoxCrossError, b5.boxdistance, b1)


if __name__ == "__main__":
    unittest.main()



import sys; sys.path.insert(0, "../..")

from pyx import *
from pyx.config import open, format
from pyx.font import T1font
from pyx.font.t1file import from_PF_bytes
from pyx.font.afmfile import AFMfile

f = T1font(from_PF_bytes(open("cmr10", [format.type1]).read()),
           AFMfile(open("cmr10", [format.afm], ascii=True)))

c = canvas.canvas()

def show(x_pt, y_pt, text, size_pt, **kwargs):
    t = f.text_pt(x_pt, y_pt, text, size_pt, **kwargs)
    c.insert(t)
    c.stroke(t.bbox().rect(), [style.linewidth(0)])

show(0, 0, "Hello, World!", 10)
show(0, -30, "Hello, World!", 10, spaced_pt=5)
show(0, -60, "Hello, World!", 10, spaced_pt=10)
show(0, -90, "Hello, World!", 10, kerning=True)

show(200, 0, "Hello, World!", 25)
show(200, -30, "Hello, World!", 25, spaced_pt=5)
show(200, -60, "Hello, World!", 25, spaced_pt=10)
show(200, -90, "Hello, World!", 25, kerning=True)

c.writeEPSfile()
c.writeEPSfile("font_direct_textaspath", write_textaspath=True)
c.writePDFfile(write_compress=False)
c.writePDFfile("font_direct_textaspath", write_textaspath=True, write_compress=False)
c.writeSVGfile(write_textaspath=False)
c.writeSVGfile("font_direct_textaspath", write_textaspath=True)



import sys; sys.path.insert(0, "../..")
from pyx import *

c = canvas.canvas()

p = path.rect(0, 0, 3, 3) + path.rect(1, 1, 1, 1)

c.fill(p)

c.fill(p, [trafo.translate(4, 0), style.fillrule.even_odd])

c.writeEPSfile(page_paperformat=document.paperformat.A4, page_fittosize=1)
c.writePDFfile(page_paperformat=document.paperformat.A4, page_fittosize=1)
c.writeSVGfile(page_paperformat=document.paperformat.A4, page_fittosize=1)




#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

c = canvas.canvas()

s = svgfile.svgfile_pt(100, 200, 'data/testsvg.svg', width_pt=450, parsed=True)
c.insert(s)
c.stroke(s.bbox().enlarged(-style.linewidth.normal.width/2).rect())
c.fill(path.circle_pt(105, 205, 5))
c.fill(path.circle_pt(105, 495, 5))
c.fill(path.circle_pt(545, 205, 5))
c.fill(path.circle_pt(545, 495, 5))

s = svgfile.svgfile_pt(100, 600, 'data/testsvg.svg', width_pt=450)
c.insert(s)
c.stroke(s.bbox().enlarged(-style.linewidth.normal.width/2).rect())
c.fill(path.circle_pt(105, 605, 5))
c.fill(path.circle_pt(105, 895, 5))
c.fill(path.circle_pt(545, 605, 5))
c.fill(path.circle_pt(545, 895, 5))

c.writeSVGfile(page_bboxenlarge=0)



import sys; sys.path[:0] = ["../.."]

from pyx import *

d = document.document()

for nr in range(1, 10):
     c = canvas.canvas()
     if nr != 7:
         c.text(0, 0, "page %d" % nr)
     d.append(document.page(c, pagename=chr(64+nr), rotated=(nr-1)%2, fittosize=1, paperformat=document.paperformat.A4))

d.writePSfile("test_document")
d.writePDFfile("test_document")




#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

import math
from pyx import *

def test_bar(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5, width=5, x=graph.axis.bar(title="Month", painter=graph.axis.painter.bar(nameattrs=[text.halign.right, trafo.rotate(90)]))))
    g.plot(graph.data.file("data/testdata2", xname=1, y=2, text=2), [graph.style.barpos(fromvalue=0), graph.style.bar(), graph.style.text()])

def test_bar2(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5, width=5, y=graph.axis.bar(title="Month")))
    g.plot(graph.data.file("data/testdata2", x=2, dx="1", yname=1), [graph.style.bar(), graph.style.errorbar()])

def test_bar3(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5, width=12, x=graph.axis.nestedbar(painter=graph.axis.painter.bar(innerticklength=0.2)), key=graph.key.key()))
    g.plot([graph.data.file("data/testdata2", xname="$1, 0", y=2),
            graph.data.file("data/testdata2", xname="$1, 1", y=3),
            graph.data.file("data/testdata2", xname="$1, 2", y=3, title=None)], [graph.style.bar()])

def test_bar4(c, x, y):
    subbarwithtext = graph.axis.bar(dist=0,
                                    painter=graph.axis.painter.bar(basepathattrs=None),
                                    linkpainter=graph.axis.painter.bar(basepathattrs=None))
    g = c.insert(graph.graphxy(x, y, height=5, width=12,
                               x=graph.axis.nestedbar(defaultsubaxis=graph.axis.bar(subaxes={'A': graph.axis.lin(painter=None, linkpainter=None, parter=None),
                                                                                             'B': graph.axis.bar(painter=graph.axis.painter.bar(basepathattrs=None),
                                                                                                                 linkpainter=graph.axis.painter.bar(basepathattrs=None))},
                                                                                    painter=graph.axis.painter.bar(basepathattrs=None),
                                                                                    linkpainter=graph.axis.painter.bar(basepathattrs=None))),
                               key=graph.key.key()))
    g.plot([graph.data.data(graph.data.points([['x',  1, 12], ['y',  2, 11], ['z',  3, 10]], id=1, y=2, ystack1=3, title="test"), xname="id, 'A'"),
            graph.data.data(graph.data.points([['x',  4,  9], ['y',  5,  8], ['z',  6,  7]], id=1, y=2, ystack1=3, title="test"), xname="id, ('B', 'X')"),
            graph.data.data(graph.data.points([['x',  7,  6], ['y',  8,  5], ['z',  9,  4]], id=1, y=2, ystack1=3, title="test"), xname="id, ('B', 'Y')"),
            graph.data.data(graph.data.points([['x', 10,  3], ['y', 11,  2], ['z', 12,  1]], id=1, y=2, ystack1=3, title="test"), xname="id, ('B', 'Z')")],
           [graph.style.barpos(fromvalue=0), graph.style.bar(), graph.style.stackedbarpos("ystack1", addontop=1), graph.style.bar([color.gradient.ReverseRainbow])])

def test_bar5(c, x, y):
    g = c.insert(graph.graphxyz(x, y, size=3, x=graph.axis.bar(), y=graph.axis.bar(), z=graph.axis.lin()))
    g.plot(graph.data.data(graph.data.points([[1, 1, 1.4], [1, 2, 1.8], [2, 1, -0.5], [2, 2, 0.9]]), xname=1, yname=2, z=3),
           [graph.style.barpos(fromvalue=0, frompathattrs=None), graph.style.bar(barattrs=[style.linejoin.bevel])])

c = canvas.canvas()
test_bar(c, 0, 0)
test_bar2(c, 7, 0)
test_bar3(c, 0, -7)
test_bar4(c, 0, -14)
test_bar5(c, 7, -19)
c.writeEPSfile("test_bargraph", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_bargraph", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_bargraph", page_paperformat=document.paperformat.A4)




from math import pi, atan2
from pyx import *

z1 = 2.5 + 4.5j
z2 = 7.5 + 5.5j

# we abuse a parametric function below, so we express everything in terms of a parameter k
x = lambda k: int(k)//11
y = lambda k: int(k)%11
z = lambda k: x(k) + y(k) * 1j
f = lambda k: 1/(z(k)-z1)/(z(k)-z2)                # function to be plotted
s = lambda k: 5*abs(f(k))                          # magnitude of function value
a = lambda k: 180/pi*atan2(f(k).imag, f(k).real)   # direction of function value

g = graph.graphxy(width=8,
                  x=graph.axis.linear(min=0, max=10),
                  y=graph.axis.linear(min=0, max=10))
g.plot(graph.data.paramfunction("k", 0, 120,
                                "x, y, size, angle = x(k), y(k), s(k), a(k)",
                                points=121, context=locals()),# access extern
       [graph.style.arrow()])                                 # variables&functions
                                                              # by passing a context
g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



#!/usr/bin/env python3
import sys;sys.path.insert(0, "..")
from math import *
from pyx import *
# File for invalid parametrisations of Bezier curves
# Draws a sketch of the areas where invalid params are to be expected

def lhs(x, y, sign1):
    if sign1 > 0:
        return -3*y*(-x + abs(x))
    else:
        return -3*y*(-x - abs(x))

def rhs(x, y, sign2):
    if sign2 > 0:
        return (1-3*x) * (1-y + sqrt(1+y+y*y))
    else:
        return (1-3*x) * (1-y - sqrt(1+y+y*y))

def f(x, y):
    xsigns = [-1, 1]
    ysigns = [-1, 1]
    if 0 < x < 1:
        xsigns = [1]
    if y > -1:
        ysigns = [-1]

    val = float("inf")
    for sign1 in xsigns:
        for sign2 in ysigns:
            val = min(val, abs(lhs(x,y,sign1) - rhs(x,y,sign2)))
    return val

xmin, xmax, xn = -2, 3, 250
xvalues = [xmin + (xmax-xmin)*i/(xn-1) for i in range(xn)]
ymin, ymax, yn = -3, 2, 250
yvalues = [ymin + (ymax-ymin)*i/(yn-1) for i in range(yn)]

d = []
for x in xvalues:
    for y in yvalues:
        d.append((x, y, log(f(x, y))))

g = graph.graphxy(width=10,
    x=graph.axis.lin(title=r"$\Delta x$", min=xmin, max=xmax),
    y=graph.axis.lin(title=r"$\Delta y$", min=ymin, max=ymax),
)
g.plot(graph.data.points(d, x=1, y=2, color=3, title=None),
       [graph.style.density(gradient=color.rgbgradient.Rainbow)])
g.dolayout()
g.plot(graph.data.function("x(y)=(-1-2*y-sqrt((1+2*y)**2+3))/3.0"), [graph.style.line([style.linestyle.dotted])])
g.plot(graph.data.function("x(y)=(-1-2*y+sqrt((1+2*y)**2+3))/3.0", max=-1), [graph.style.line([style.linestyle.dotted])])
g.writePDFfile()




import sys
if sys.path[0] != "../..":
    sys.path.insert(0, "../..")

import unittest

from pyx import *
from pyx.graph.axis.tick import rational


class RationalTestCase(unittest.TestCase):

    def RationalEqual(self, num, denom, r):
        self.assertEqual(num*r.denom, r.num*denom)

    def testRationalInitSeq(self):
        self.RationalEqual(1, 1, rational((2, 2)))
        self.RationalEqual(1, 1, rational([2, 2]))

    def testRationalInitString(self):
        self.RationalEqual(1, 1, rational("1"))
        self.RationalEqual(11, 10, rational("1.1"))
        self.RationalEqual(12345, 1000, rational("12.345"))
        self.RationalEqual(1, 1, rational("1."))
        self.RationalEqual(1, 10, rational(".1"))
        self.RationalEqual(1, 1, rational("1e+0"))
        self.RationalEqual(11, 10, rational("1.1e-0"))
        self.RationalEqual(10, 1, rational("1.e+1"))
        self.RationalEqual(1, 100, rational(".1e-1"))
        self.RationalEqual(-1, 1, rational("-1"))
        self.RationalEqual(-11, 10, rational("-1.1"))
        self.RationalEqual(-1, 1, rational("-1."))
        self.RationalEqual(-1, 10, rational("-.1"))
        self.RationalEqual(-1, 1, rational("-1e0"))
        self.RationalEqual(-11, 10, rational("-1.1e-0"))
        self.RationalEqual(-10, 1, rational("-1.e+1"))
        self.RationalEqual(-1, 100, rational("-.1e-1"))
        self.RationalEqual(-100000000000000000000, 1, rational("-1e+20"))
        self.RationalEqual(-1, 100000000000000000000, rational("-1e-20"))

        self.RationalEqual(1234, 1, rational(" 1234"))
        self.assertRaises(ValueError, rational, "12 34")
        self.assertRaises(ValueError, rational, "1 2.34")
        self.assertRaises(ValueError, rational, "12 .34")
        self.assertRaises(ValueError, rational, "12. 34")
        self.assertRaises(ValueError, rational, "12.3 4")
        self.RationalEqual(1234, 100, rational("12.34 "))
        self.RationalEqual(1234, 1, rational(" +1234"))
        self.RationalEqual(-1234, 1, rational(" -1234"))
        self.assertRaises(ValueError, rational, " + 1234")
        self.assertRaises(ValueError, rational, " - 1234")
        self.assertRaises(ValueError, rational, "12.34 e0")
        self.assertRaises(ValueError, rational, "12.34e 0")
        self.RationalEqual(1234, 10000, rational("12.34e-2"))
        self.RationalEqual(1234, 1000, rational("12.34E-1"))
        self.RationalEqual(1234, 100, rational("12.34e0 "))
        self.RationalEqual(1234, 10, rational("12.34E+1"))
        self.RationalEqual(1234, 1, rational("12.34e+2"))
        self.assertRaises(ValueError, rational, "12.34e -0")
        self.assertRaises(ValueError, rational, "12.34e+ 0")
        self.assertRaises(ValueError, rational, "12.34e- 0")
        self.assertRaises(ValueError, rational, "12.34e +0")
        self.assertRaises(ValueError, rational, "12.34.56")
        self.assertRaises(ValueError, rational, "12e34.56")

    def testRationalInitStrings(self):
        self.RationalEqual(1, 2, rational("1/2"))
        self.RationalEqual(1, 2, rational("1.1/2.2"))
        self.RationalEqual(1, 2, rational("1./2."))
        self.RationalEqual(1, 2, rational(".1/.2"))
        self.RationalEqual(1, 2, rational("1e+0/2e+0"))
        self.RationalEqual(1, 2, rational("1.1e-0/2.2e-0"))
        self.RationalEqual(1, 2, rational("1.e+1/2.e+1"))
        self.RationalEqual(1, 2, rational(".1e-1/.2e-1"))
        self.RationalEqual(1, 2, rational("-1/-2"))
        self.RationalEqual(1, 2, rational("-1.1/-2.2"))
        self.RationalEqual(1, 2, rational("-1./-2."))
        self.RationalEqual(1, 2, rational("-.1/-.2"))
        self.RationalEqual(1, 2, rational("-1e0/-2e0"))
        self.RationalEqual(1, 2, rational("-1.1e-0/-2.2e-0"))
        self.RationalEqual(1, 2, rational("-1.e+1/-2.e+1"))
        self.RationalEqual(1, 2, rational("-.1e-1/-.2e-1"))

    def testRationalInitNumber(self):
        self.RationalEqual(1, 1, rational(1))
        self.RationalEqual(11, 10, rational(1.1))
        self.RationalEqual(1, 1, rational(1.))
        self.RationalEqual(1, 10, rational(.1))
        self.RationalEqual(1, 1, rational(1e+0))
        self.RationalEqual(11, 10, rational(1.1e-0))
        self.RationalEqual(10, 1, rational(1.e+1))
        self.RationalEqual(1, 100, rational(.1e-1))
        self.RationalEqual(-1, 1, rational(-1))
        self.RationalEqual(-11, 10, rational(-1.1))
        self.RationalEqual(-1, 1, rational(-1.))
        self.RationalEqual(-1, 10, rational(-.1))
        self.RationalEqual(-1, 1, rational(-1e0))
        self.RationalEqual(-11, 10, rational(-1.1e-0))
        self.RationalEqual(-10, 1, rational(-1.e+1))
        self.RationalEqual(-1, 100, rational(-.1e-1))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-

from setuptools import setup, Extension
import configparser

cfg = configparser.ConfigParser()
cfg.read("setup.cfg")

# build list of extension modules

ext_modules = []
pykpathsea_ext_module = Extension("pyx.pykpathsea",
                                  sources=["pyx/pykpathsea.c"],
                                  libraries=["kpathsea"])
t1code_ext_module = Extension("pyx.font._t1code",
                              sources=["pyx/font/_t1code.c"])
if cfg.has_option("PyX", "build_pykpathsea") and cfg.getboolean("PyX", "build_pykpathsea"):
    ext_modules.append(pykpathsea_ext_module)
if cfg.has_option("PyX", "build_t1code") and cfg.getboolean("PyX", "build_t1code"):
    ext_modules.append(t1code_ext_module)

setup(ext_modules=ext_modules)



import sys, re
sys.path[:0] = [".."]
from pyx import *

text.set(text.LatexRunner)
text.preamble(r"\renewcommand{\familydefault}{\ttdefault}")

c = canvas.canvas()

# data to be plotted
pf = graph.data.paramfunction("k", 0, 1, "color, xmin, xmax, ymin, ymax= k, k, 1, 0, 1")

# positioning is quite ugly ... but it works at the moment
y = 0
dy = -0.65

# we could use gradient.__dict__ to get the instances, but we
# would loose the ordering ... instead we just parse the file:

# see comment in colorname.py

firstgraph = None
p = re.compile("(?P<id>gradient\\.(?P<name>[a-z]+)) += [a-z]*gradient_[a-z]+\\(", re.IGNORECASE)
with open(color.__file__) as f:
    for line in f:
        m = p.match(line)
        if m:
            if firstgraph is None:
                xaxis = graph.axis.lin(
                    parter=graph.axis.parter.lin(tickdists=["0.5","0.1"], labeldists=["1"]),
                    painter=graph.axis.painter.regular(
                        innerticklength=None,
                        outerticklength=graph.axis.painter.ticklength.normal),
                    linkpainter=graph.axis.painter.regular(innerticklength=None, labelattrs=None))
                firstgraph = g = graph.graphxy(ypos=y, width=10, height=0.5, x2=xaxis, y=graph.axis.lin(parter=None))
            else:
                g = graph.graphxy(ypos=y, width=10, height=0.5, x2=graph.axis.linkedaxis(firstgraph.axes["x2"]), y=graph.axis.lin(parter=None))
            g.plot(pf, [graph.style.rect(gradient=getattr(color.gradient, m.group("name")), keygraph=None)])
            g.doplot()
            g.finish()
            c.insert(g)
            c.text(10.2, y + 0.15, m.group("id"), [text.size.footnotesize])
            y += dy


c.writePDFfile()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

c = canvas.canvas()

p = ( path.line(10*unit.u_pt, 10*unit.u_pt, 40*unit.u_pt, 40*unit.u_pt) +
      path.line(10*unit.u_pt, 40*unit.u_pt, 40*unit.u_pt, 10*unit.u_pt) )

t1 = trafo.rotate(20)
t2 = trafo.translate(5,0)
t3 = trafo.mirror(10)

sc = canvas.canvas([t1, t2, t3])
c.insert(sc).stroke(p)

c.stroke(c.bbox().rect())

c.stroke(p.transformed(t1*t2*t3), [color.rgb.green, style.linestyle.dashed])

c.stroke(p, [color.rgb.red, style.linestyle.dotted, t3, t2, t1])

c.writeEPSfile("test_canvas", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_canvas", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_canvas", page_paperformat=document.paperformat.A4)




from pyx import *

c = canvas.canvas()
c.fill(path.rect(0, 0, 7, 3), [color.gray(0.8)])
c.fill(path.rect(1, 1, 1, 1), [color.rgb.red])
c.fill(path.rect(3, 1, 1, 1), [color.rgb.green])
c.fill(path.rect(5, 1, 1, 1), [color.rgb.blue])
c.writePDFfile()




import sys; sys.path.insert(0, "../..")
from glob import glob
from pyx import *
from PIL import Image

d = document.document()
for i, f in enumerate(glob('suite/svg/shapes-*.svg') +
                      glob('suite/svg/paths-data-*.svg')):
    #if i + 1 != 15: continue
    print(i+1, f)
    c = canvas.canvas()
    i = Image.open(f.replace('svg', 'png'))
    b = bitmap.bitmap(0, 0, i)
    c.insert(b)
    c.insert(svgfile.svgfile(b.bbox().width(), 0, f, parsed=True, resolution=i.info["dpi"][0]))
    d.append(document.page(c))
d.writePDFfile()



import sys; sys.path.insert(0, "../..")
from pyx import *

try:
    enumerate([])
except NameError:
    # fallback implementation for Python 2.2 and below
    def enumerate(list):
        return list(zip(list(range(len(list))), list))


c = canvas.canvas()

# color conversion tests
def UCRc(x): return x
def UCRm(x): return x
def UCRy(x): return x
def BG(x): return x
color.set(UCRc=UCRc, UCRm=UCRm, UCRy=UCRy, BG=BG)

def colrow(can, pos, acol):
  width, height = 1.0, 1.0
  colors = [acol.grey(), acol.rgb(), acol.hsb(), acol.cmyk()]
  for i, col in enumerate(colors):
      can.draw(path.rect(pos[0]+i*width, pos[1], width, height), [deco.filled([col]), deco.stroked()])
  return pos[0], pos[1] - height

pos = 0, -5
for col in [color.grey.black, color.grey(0.25), color.grey(0.5), color.grey(0.75), color.grey.white,
            color.rgb.red, color.rgb.green, color.rgb.blue, color.rgb.white, color.rgb.black,
            color.cmyk.GreenYellow, color.cmyk.Yellow, color.cmyk.Goldenrod, color.cmyk.Dandelion, color.cmyk.Apricot,
            color.cmyk.Peach, color.cmyk.Melon, color.cmyk.YellowOrange, color.cmyk.Orange, color.cmyk.BurntOrange,
            color.cmyk.Bittersweet, color.cmyk.RedOrange, color.cmyk.Mahogany, color.cmyk.Maroon, color.cmyk.BrickRed,
            color.cmyk.Red, color.cmyk.OrangeRed, color.cmyk.RubineRed, color.cmyk.WildStrawberry, color.cmyk.Salmon,
            color.cmyk.CarnationPink, color.cmyk.Magenta, color.cmyk.VioletRed, color.cmyk.Rhodamine, color.cmyk.Mulberry,
            color.cmyk.RedViolet, color.cmyk.Fuchsia, color.cmyk.Lavender, color.cmyk.Thistle, color.cmyk.Orchid,
            color.cmyk.DarkOrchid, color.cmyk.Purple, color.cmyk.Plum, color.cmyk.Violet, color.cmyk.RoyalPurple,
            color.cmyk.BlueViolet, color.cmyk.Periwinkle, color.cmyk.CadetBlue, color.cmyk.CornflowerBlue,
            color.cmyk.MidnightBlue, color.cmyk.NavyBlue, color.cmyk.RoyalBlue, color.cmyk.Blue, color.cmyk.Cerulean,
            color.cmyk.Cyan, color.cmyk.ProcessBlue, color.cmyk.SkyBlue, color.cmyk.Turquoise, color.cmyk.TealBlue,
            color.cmyk.Aquamarine, color.cmyk.BlueGreen, color.cmyk.Emerald, color.cmyk.JungleGreen, color.cmyk.SeaGreen,
            color.cmyk.Green, color.cmyk.ForestGreen, color.cmyk.PineGreen, color.cmyk.LimeGreen, color.cmyk.YellowGreen,
            color.cmyk.SpringGreen, color.cmyk.OliveGreen, color.cmyk.RawSienna, color.cmyk.Sepia, color.cmyk.Brown, color.cmyk.Tan,
            color.cmyk.Gray, color.cmyk.Black, color.cmyk.White, color.cmyk.white, color.cmyk.black]:
    pos = colrow(c, pos, col)


c.writeEPSfile("test_color", page_paperformat=document.paperformat.A4, page_fittosize=1)

# transparency tests
c.fill(path.rect(-1, -1, 2, 2), [color.rgb.red])
c.fill(path.circle(0, 0, 1.2), [color.transparency(0.5), color.rgb.green])
c.fill(path.rect(-2, -0.5, 4, 1), [color.transparency(0.9), color.rgb.blue])

c.writePDFfile("test_color", page_paperformat=document.paperformat.A4, page_fittosize=1)
c.writeSVGfile("test_color", page_paperformat=document.paperformat.A4, page_fittosize=1)




import sys; sys.path.insert(0, "../..")
from pyx import *
from solve import scalar, vector, solver

A = vector([0, 0], "A")
B = vector([10, 5], "B")
C = vector([0, 10], "C")
D = vector([scalar(), 0], "D")

solver.eq((B-A)*(D-C), 0)

def line(p1, p2):
    return path.line(float(p1.x), float(p1.y), float(p2.x), float(p2.y))

c = canvas.canvas()

c.stroke(line(A, B))
c.stroke(line(C, D))

c.writeEPSfile()
c.writePDFfile()




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
#
# Copyright (C) 2005 André Wobst <wobsta@pyx-project.org>
#
# epstopng is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# epstopng is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with epstopng; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from optparse import OptionParser
from pyx import *
from pyx import version

parser = OptionParser(usage="usage: %prog -o output-file [-r resolution] image-file",
                      version="%prog " + version.version)
parser.add_option("-o", "--output",
                  type="string", dest="output",
                  help="output-file")
parser.add_option("-r", "--resolution",
                  type="string", dest="resolution", default=None,
                  help="resolution of the image in dpi (optional when available in the image data)")
(options, args) = parser.parse_args()
if len(args) != 1:
    parser.error("can process a single image-file only")

try:
    im = bitmap.jpegimage(args[0])
except ValueError:
    from PIL import Image
    im = Image.open(args[0])
    compressmode = "Flate"
else:
    compressmode = None
if options.resolution is None:
    width = None
else:
    width = im.size[0] / float(options.resolution) * unit.inch
c = canvas.canvas()
c.insert(bitmap.bitmap(0, 0, im, compressmode=compressmode, width=width))
c.writetofile(options.output)



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

import math
from pyx import *

text.set(text.LatexEngine)

def test_multiaxes_data(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5, key=graph.key.key(pos="tl"),
                               x=graph.axis.log(title="$W$", manualticks=[graph.axis.tick.tick(math.sqrt(8)*100, label="?"), graph.axis.tick.tick(math.sqrt(8), label=r"$\sqrt{8}$")]),
                               y=graph.axis.log(title=r"$PPP_1$",
                                               painter=graph.axis.painter.regular(titledirection=None)),
                               y2=graph.axis.log(title="$P_2$"),
                               y3=graph.axis.log(title="$PPP_3$",
                                                painter=graph.axis.painter.regular(titledirection=graph.axis.painter.rotatetext(45), gridattrs=[color.gradient.RedGreen]),
                                                texter=graph.axis.texter.decimal(equalprecision=1)),
                               y5=graph.axis.log(title="$P_5$")))
    g.plot((graph.data.file("data/testdata", x=1, y="sqrt(sqrt($3))", title="mytitle"),
            graph.data.file("data/testdata", x=1, y2=4),
            graph.data.file("data/testdata", x=1, y3=5, title=None),
            graph.data.file("data/testdata", x=1, y5=6)),
           styles=[graph.style.symbol(symbolattrs=[deco.stroked.clear, color.gradient.RedGreen, graph.style.symbol.changestrokedfilled], symbol=graph.style.symbol.changesquaretwice)])
    g.finish()

def test_piaxis_function(c, x, y):
    xaxis=graph.axis.lin(min=0, max=2*math.pi, divisor=math.pi, texter=graph.axis.texter.rational(suffix=r"\pi"))
    g = c.insert(graph.graphxy(x, y, height=5, x=xaxis))
    #g = c.insert(graph.graphxy(x, y, height=5, x=xaxis, x2=xaxis))
    g.plot([graph.data.function("y(x)=sin(x-i*pi/10)", context={"i": i}) for i in range(20)],
           styles=[graph.style.line(lineattrs=[color.gradient.Hue])])
    g.finish()

def test_textaxis_errorbars(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5,
                               x=graph.axis.lin(min=0.5, max=12.5, parter=graph.axis.parter.lin("1", extendtick=None)),
                               y=graph.axis.lin(min=-10, max=30, title=r"Temperature [$^\circ$C]"),
                               x2=graph.axis.lin(painter=graph.axis.painter.regular(labelattrs=None)), y2=graph.axis.lin()))
    g.plot(graph.data.file("data/testdata2", x=0, ymin="min", ymax="max"), [graph.style.errorbar()])
    a = graph.style.symbol.triangle
    g.plot(graph.data.paramfunction("k", 0, 2*math.pi, "x2, y2, dx2, dy2 = 0.8*sin(k), 0.8*cos(3*k), 0.05, 0.05"), [graph.style.symbol(symbol=a), graph.style.errorbar()])
    g.finish()

def test_ownmark(c, x, y):
    div = lambda x, y: int(x)/int(y)
    mod = lambda x, y: int(x)%int(y)
    g = c.insert(graph.graphxy(x, y, height=5, x=graph.axis.lin(min=0, max=10), y=graph.axis.lin(min=0, max=10)))
    g.plot(graph.data.paramfunction("k", 0, 120, "x, y, size, angle = mod(k, 11), div(k, 11), (1+sin(k*pi/120))/2, 3*k", points=121, context=locals()), [graph.style.arrow()])
    line1 = g.plot(graph.data.function("y(x)= 10/x"))
    line2 = g.plot(graph.data.functionxy(lambda x: 12*x**-1.6))
    line3 = g.plot(graph.data.function("y(x)=7/x"))
    g.dolayout()
    line4 = g.plot(graph.data.function("y(x)=25*x**-1.6"))
    g.plot(graph.data.points([[-1, 1], [5, 2], [11, 5], [5, 11], [4, -1]], x=1, y=2), [graph.style.line(lineattrs=[color.rgb.red])])
    g.finish()

    p1=line1.path
    p2=line2.path.reversed()
    p3=line3.path.reversed()
    p4=line4.path
    (seg1a,), (seg2a,) = p1.intersect(p2)
    (seg2b,), (seg3b,) = p2.intersect(p3)
    (seg3c,), (seg4c,) = p3.intersect(p4)
    (seg4d,), (seg1d,) = p4.intersect(p1)
    area = p1.split([seg1a, seg1d])[1] << p4.split([seg4d, seg4c])[1] << p3.split([seg3c, seg3b])[1] << p2.split([seg2b, seg2a])[1]
    area.normsubpaths[-1].close()
    g.stroke(area, [style.linewidth.THick, deco.filled([color.gray(0.5)])])

def test_allerrorbars(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5, width=5))
    g.plot(graph.data.file("data/testdata3", x="x", y="y", xmin="xmin", xmax="xmax", ymin="ymin", ymax="ymax", text="text"), [graph.style.text(), graph.style.errorbar(), graph.style.symbol()])
    g.finish()

def test_split(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5, width=5,
                               x=graph.axis.log(),
                               y=graph.axis.split()))
    g.plot(graph.data.file("data/testdata", x=1, y="splitatvalue($3, 0.005, 0.01, 0.015, 0.018)"))
    g.finish()

def test_split2(c, x, y):
    g = c.insert(graph.graphxy(x, y, height=5, width=5,
                               x=graph.axis.log(),
                               y=graph.axis.split(subaxes=[graph.axis.sizedlinear(size=0.005), graph.axis.sizedlinear(size=0.0025), graph.axis.autosizedlinear()], dist=0.0025)))
    g.plot(graph.data.file("data/testdata", x=1, y="splitatvalue(0.0215-$3, 0.005, 0.01, 0.015, 0.015)"))
    g.finish()


c = canvas.canvas()
test_multiaxes_data(c, 0, 21)
test_piaxis_function(c, 0, 14)
test_textaxis_errorbars(c, 0, 7)
test_ownmark(c, 0, 0)
test_allerrorbars(c, -7, 0)
test_split(c, -7, 7)
test_split2(c, -7, 14)

c.writeEPSfile("test_graph", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_graph", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_graph", page_paperformat=document.paperformat.A4)




#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *
from pyx.path import *

def bboxrect(cmd):
    return cmd.bbox().rect()

def drawpathwbbox(c, p):
    c.stroke(p, [color.rgb.red])
    np = p.normpath()
    c.stroke(np, [color.rgb.green, style.linestyle.dashed])
    c.stroke(bboxrect(p))

def testarcbbox(c):
    for phi in range(0,360,30):
       drawpathwbbox(c,path(arc(phi*0.1, phi*0.1, 1, 0, phi)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(arc(phi*0.1, 5+phi*0.1, 1, phi, 360)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(arc(phi*0.1, 10+phi*0.1, 1, phi, phi+30)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(arc(phi*0.1, 15+phi*0.1, 1, phi, phi+120)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(arc(phi*0.1, 20+phi*0.1, 1, phi, phi+210)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(arc(phi*0.1, 25+phi*0.1, 1, phi, phi+300)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(arc(phi*0.1, 30+phi*0.1, 1, phi, phi+390)))
       

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(20+phi*0.1, phi*0.09),
                            arc(20+phi*0.1, phi*0.1, 1, 0, phi)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(20+phi*0.1, 5+phi*0.11),
                            arc(20+phi*0.1, 5+phi*0.1, 1, 0, phi)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(20+phi*0.1, 10+phi*0.09),
                            arcn(20+phi*0.1, 10+phi*0.1, 1, 0, phi)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(20+phi*0.1, 15+phi*0.11),
                            arcn(20+phi*0.1, 15+phi*0.1, 1, 0, phi)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(50+phi*0.1, phi*0.09),
                            arc(50+phi*0.1, phi*0.1, 1, 0, phi),
                            rlineto(1,1)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(50+phi*0.1, 5+phi*0.11),
                            arc(50+phi*0.1, 5+phi*0.1, 1, 0, phi),
                            rlineto(1,1)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(50+phi*0.1, 10+phi*0.09),
                            arcn(50+phi*0.1, 10+phi*0.1, 1, 0, phi),
                            rlineto(1,1)))

    for phi in range(0,360,30):
       drawpathwbbox(c,path(moveto(50+phi*0.1, 15+phi*0.11),
                            arcn(50+phi*0.1, 15+phi*0.1, 1, 0, phi),
                            rlineto(1,1)))


def testcurvetobbox(c):
    drawpathwbbox(c,path(moveto(10,60), curveto(12,66,14,65,12,69)))


def testtrafobbox(c):
    sc=c.insert(canvas.canvas([trafo.translate(0,40).rotated(10)]))

    p=path(moveto(10,10), curveto(12,16,14,15,12,19));   drawpathwbbox(sc,p)
    p=path(moveto(5,17), curveto(6,18, 5,16, 7,15));     drawpathwbbox(sc,p)


def testclipbbox(c):
    clip = canvas.clip(rect(11,-9,10,5))

    p1 = path(moveto(10,-10), curveto(12,-4, 14,-5, 12,-1));   
    p2 = path(moveto(12,-8), curveto(6,-2, 5,-4, 7,-5));  
    
    # just a simple test for clipping
    sc = c.insert(canvas.canvas([clip]))
    drawpathwbbox(sc, p1)
    drawpathwbbox(sc, p2)

    # more complicated operations
    
    # 1. transformation followed by clipping:
    # in this case, the clipping path will be evaluated in the
    # context of the already transformed canvas, so that the
    # actually displayed portion of the path should be the same
    sc = c.insert(canvas.canvas([trafo.translate(5,0), clip]))
    drawpathwbbox(sc, p1)
    drawpathwbbox(sc, p2)

    # 2. clipping followed by transformation 
    # in this case, the clipping path will not be transformed, so
    # that the display portionof the path should change
    sc = c.insert(canvas.canvas([clip, trafo.translate(1,1)]))
    drawpathwbbox(sc, p1)
    drawpathwbbox(sc, p2)


c = canvas.canvas()
testarcbbox(c)
testcurvetobbox(c)
testtrafobbox(c)
testclipbbox(c)
c.writeEPSfile("test_bbox", page_paperformat=document.paperformat.A4, page_rotated=1, page_fittosize=1)
c.writePDFfile("test_bbox")
c.writeSVGfile("test_bbox")
